/* copyright © by Stefan Voitel, 2003 - all rights reserved                   */
/* eMail : stefan.voitel@winways.de                                           */
/* a simple phrase (joke , quote) generator                                   */

#ifndef __CLASS_PHRASE_INCLUDED__
#define __CLASS_PHRASE_INCLUDED__

#include <string>

#ifndef to_wchar
#define to_wchar 1
CHAR* toWChar(std::string str);
WCHAR* toWCharR(std::string str);
#endif

#include <windows.h>

class cPhrase {

  public:
         cPhrase     (char *File);
        ~cPhrase     (void);

   char *Get         (void);

  private:

   char          *Text;
   char         **Lines;
   unsigned int   LineCount;
   unsigned int  *Phrases;
   unsigned int   PhraseCount;
   unsigned int   CurrentLine;

   bool           IsEmptyLine       (char* Line);
   void           SeparateLines     (void);
   void           SeparatePhrases   (void);
};

/*****************************************************************************/
// Implementation
/*****************************************************************************/

cPhrase::cPhrase     (char *File) {
CurrentLine    = 0xFFFFFFFF;
Text           = NULL;
Lines          = NULL;
LineCount      = 0;
Phrases        = NULL;
PhraseCount    = 0;

HANDLE hf;

if ((hf = CreateFile (toWChar(File),GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,
                                             0,NULL)) != INVALID_HANDLE_VALUE) {
   DWORD Size = GetFileSize (hf,NULL), rd;

   if ((Text = (char*) GlobalAlloc (GPTR,(Size+1)*sizeof(char))) != NULL) {
      if (ReadFile(hf,Text,Size,&rd,NULL) && rd == Size) {
         SeparatePhrases ();
         }
      }
   CloseHandle (hf);
   }
}

cPhrase::~cPhrase    (void) {

if (Phrases != NULL) {
   GlobalFree((HGLOBAL) Phrases);
   }

if (Lines != NULL) {
   GlobalFree((HGLOBAL) Lines);
   }

if (Text != NULL) {
   GlobalFree((HGLOBAL) Text);
   }
}

// get a line of a random phrase if the line empty return NULL
// to indicate the end has reached
char *cPhrase::Get               (void) {
char  *Line = NULL;

if (LineCount && PhraseCount && Lines != NULL && Phrases != NULL) {

   if (CurrentLine == 0xFFFFFFFF) {
      srand (GetTickCount());
      unsigned int random = rand();
      CurrentLine = Phrases [random % PhraseCount];
      }

   if (IsEmptyLine (Lines[CurrentLine])) {
      CurrentLine = 0xFFFFFFFF;
      }
   else if (CurrentLine < LineCount) {
      Line = Lines[CurrentLine ++];
      }
   }
return Line;
}

bool  cPhrase::IsEmptyLine       (char* Line) {
bool empty = true;
char *p = Line;
while (*p && *p != '\n') {
   if (*p != ' ' && *p != '\t' && *p != '\r') {
      empty = false;
      break;
      }
   p++;
   }
return empty;
}

// separate the lines by newline char, store the ptrs to every line in Lines
void  cPhrase::SeparateLines     (void) {
unsigned int Count = 0;
char        *cp = Text;

while (*cp) {
   if (*cp == '\n')
      Count++;
   cp++;
   }

if (Count > 0) {
   if ((Lines = (char**) GlobalAlloc(GPTR,(Count +1)*sizeof(char*))) != NULL) {
      char *p = Lines[0] = Text;

      while (*p) {
         if (*p == '\r')
            *p++ = ' ';

         if (*p == '\n') {
            *p = '\0';
            Lines[++ LineCount] = p+1;
            }
         p++;
         }
      }
   }
}

// separate phrases by empty lines, store ptrs in Phrases
// any ptr in Phrases marks a start of a phrase
void  cPhrase::SeparatePhrases   (void) {
SeparateLines();

int Count = 0;
for (unsigned int i = 0 ; i < LineCount ; i++) {
   if (IsEmptyLine (Lines[i]))
      Count ++;
   }

if (Count) {
   if ((Phrases = (unsigned int*) GlobalAlloc (GPTR,(Count+1) *
                                             sizeof(unsigned int))) != NULL) {
      for (unsigned int i = 0 ; i < LineCount ; i++) {
         if (IsEmptyLine (Lines[i])) {
            Phrases[++PhraseCount] = i+1;
            }
         }
      }
   }
}

#endif // (__CLASS_PHRASE_INCLUDED__)
//  ---  EOF  ---
