#ifndef JELIZA
#define JELIZA 1


/*
 * This is part of JEliza 2.3.
 * Copyright 2006 by Tobias Schulz
 * WWW: http://jeliza.ch.to/
 *
 * JEliza is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * JEliza is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU GPL
 * along with JEliza (file gpl.txt) ; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "defs.h"

using namespace std;
using jeliza::log;

/*
 * Benutze die Online DB?
 */
bool jeliza::use_online_db = true;
jeliza::string online_db_new_entry_url("http://jeliza.kilu.de/developer/online-db/new.php?");
jeliza::string online_db_new_sentence_entry_url("http://jeliza.kilu.de/developer/online-db/new_sentence.php?");

/*
 * Ein besseres "pair"-Objekt als in der Standardbibliothek
 */
namespace jeliza {
    template <typename T1, typename T2>
    class Pair {
    public:
        T1 first;
        T2 second;

        Pair (T1 f) {
            first = f;
        }
        Pair (T2 s) {
            second = s;
        }
        Pair& operator= (const Pair& p) {
            first = p.first;
            second = p.second;

            return (*this);
        }
        bool operator== (const Pair& p) {
            return (first == p.first && second == p.second);
        }
    };
}


/*
 * Logdatei oeffnen
 *
 * ofstream - output file stream
 */
ofstream clogger("jeliza.log");

/*
 * Ist die Python-Initialisierung abgeschlossen?
 */
bool jeliza::is_python_initialized = false;

/*
 * Damit es ein Objekt gibt für die spo_split-Methoden
 *
 * Dies ist ein Singleton!
 */
jeliza::spo_split jeliza::spo_split::instance;

/*
 * Auf eine bestimmte Antwort bestehen, in "Notfaellen"?
 */
bool jeliza::force_answer_check = false;
jeliza::string jeliza::force_answer;
jeliza::string jeliza::force_answer_frage;
string jeliza::force_answer_word;
string jeliza::force_answer_genus;

/*
 * Globale Initialisierung abgeschlossen?
 */
bool jeliza::global_fertig = false;

/*
 * Globale JEliza-Implementierung
 */
jeliza::jeliza_impl jeliza_impl_instance;

/*
 * Naechste Wordtype von GUI
 */
std::string jeliza::next_word_type("-1");
std::string jeliza::next_genus("-1");

/*
 * Globale Initialisierungs-Funktion
 */
void jeliza::jeliza_impl::init_databases() {
    log("Loading brain...");

    long max_size = 210000;

    /*
     * Datenbank leeren
     */
    if (jeliza::spo_split::get().Wordtype_Database != NULL)
        delete jeliza::spo_split::get().Wordtype_Database;
    jeliza::spo_split::get().Wordtype_Database = new vector<jeliza::strings>();
    jeliza::spo_split::get().Wordtype_equal.reset(new vector<Pair<std::string, jeliza::string> >());
    jeliza::spo_split::get().Wordtype_ohne_prefix.reset(new vector<Pair<std::string, jeliza::string> >());
    jeliza::spo_split::get().Wordtype_equal_low.reset(new vector<Pair<std::string, jeliza::string> >());
    jeliza::spo_split::get().Wordtype_ohne_prefix_low.reset(new vector<Pair<std::string, jeliza::string> >());
    jeliza::spo_split::get().genus_index.reset(new map<std::string, std::string>);
    jeliza::spo_split::get().genuss.reset(new vector<std::string>);

    /*
     * Benoetigten Speicherplatz ausrechnen und reservieren
     */
    long long required_space = 5 * 2 * max_size * sizeof(Pair<std::string, jeliza::string>);
    std::stringstream sst;
    std::string req;
    sst << required_space / 1024 / 1024;
    sst >> req;
    log("Reserving space for brain: " + req + " MiB");

    jeliza::spo_split::get().Wordtype_Database->reserve(max_size * 2);
    jeliza::spo_split::get().Wordtype_equal->reserve(max_size * 2);
    jeliza::spo_split::get().Wordtype_equal_low->reserve(max_size * 2);
    jeliza::spo_split::get().Wordtype_ohne_prefix->reserve(max_size * 2);
    jeliza::spo_split::get().Wordtype_ohne_prefix_low->reserve(max_size * 2);

    /*
     * Datenbank einlesen
     */
    log("Lade Gehirn aus Datei...");
    ifstream iDic(("lang_" + jeliza::get_lang_str() + "/wsh/dic.wsh").ref().c_str());
    jeliza::string str;
    int k = 0;
    unsigned int prozent;
    while (iDic) {
        ++k;
        if (k % 3000 == 0) {
            prozent = (int)(100.0 / 90000.0 * k);
            cout << "\r" << prozent << "% geladen                                            \r" << flush;
            jeliza::update_prozent(prozent);
        }

        getline(iDic, str.ref());
        jeliza::strings vec;

        util::split(str, std::string(";"), vec);
        if (vec.size() > 2) {
            add_wortschatz_word(vec);
        }
    }

    cout << endl;

    ifstream iGenus(("lang_" + jeliza::get_lang_str() + "/wsh/genus.wsh").ref().c_str());
    while (iGenus) {
        ++k;
        if (k % 3000 == 0) {
            prozent = (unsigned int)(100.0 / 90000.0 * (double)(k));
            cout << "\r" << prozent << "% geladen                                            \r" << flush;
            jeliza::update_prozent(prozent);
        }

        getline(iGenus, str.ref());
        jeliza::strings vec;

        util::split(str, std::string(";"), vec);
        if (vec.size() == 2) {
            add_wortschatz_genus(vec);
        }
    }

    /*
     * Satzauftrenndatenbank initialisieren
     */
    jeliza::spo_split::get().init_sentences_splitted();

    /*
     * Wissensdatenbank initialiseren
     */
    init();
}

void jeliza::jeliza_impl::start() {
    jeliza_impl_instance = *this;

    /*
     * Python und Python-Threads aktivieren, das C++-Modul registrieren
     */
    Py_Initialize();
    PyEval_InitThreads();
    PyImport_AddModule("jelizacpp");
    Py_InitModule("jelizacpp", jelizacpp_methods);

    /*
     * Python ist initialisiert
     */
    jeliza::is_python_initialized = true;

    /*
     * Datenbanken initialiseren
     */
    init_databases();

    /*
     * Server starten
     */
    PyRun_SimpleString("execfile(\"modules/_server.py\")");

    jeliza::global_fertig = true;
}

/*
 * Ein Wort in die Wortschatz-Datenbank eintragen
 */
void jeliza::jeliza_impl::add_wortschatz_word(jeliza::strings vec) {
    vec[0].lower();
    vec[2].strip();
    vec[3].strip();

    jeliza::strings vec_tmp;

    jeliza::spo_split::get().Wordtype_Database->push_back(vec);
    jeliza::spo_split::get().Wordtype_equal->push_back(Pair<std::string, jeliza::string>(vec[1].ref()));
    jeliza::spo_split::get().Wordtype_equal->push_back(Pair<std::string, jeliza::string>(vec[0]));
    jeliza::spo_split::get().Wordtype_ohne_prefix->push_back(Pair<std::string, jeliza::string>(vec[3].ref()));
    jeliza::spo_split::get().Wordtype_ohne_prefix->push_back(Pair<std::string, jeliza::string>(vec[0]));

    jeliza::spo_split::get().Wordtype_equal_low->push_back(Pair<std::string, jeliza::string>(vec[1].get_lower().ref()));
    jeliza::spo_split::get().Wordtype_equal_low->push_back(Pair<std::string, jeliza::string>(vec[0]));
    jeliza::spo_split::get().Wordtype_ohne_prefix_low->push_back(Pair<std::string, jeliza::string>(vec[3].get_lower().ref()));
    jeliza::spo_split::get().Wordtype_ohne_prefix_low->push_back(Pair<std::string, jeliza::string>(vec[0]));
}

void jeliza::jeliza_impl::add_wortschatz_genus(jeliza::strings vec) {
    vec[0].lower();
    vec[1].strip();

    jeliza::strings vec_tmp;

    (*jeliza::spo_split::get().genus_index)[vec[1].ref()] = vec[0].ref();
    jeliza::spo_split::get().genuss->push_back(vec[1].ref());
}

/*
 * Ein Wort in die Datenbank eintragen, nach erzwungener Antwort
 */
bool jeliza::jeliza_impl::add_word(char ch) {
    jeliza::string toWrite = jeliza::string(";" + jeliza::force_answer_word + ";" + spo_split::get().ermittle_wortstamm(jeliza::string(jeliza::force_answer_word)).ref() + ";" + spo_split::get().entferne_prefixe(jeliza::string(jeliza::force_answer_word)).ref());
    unsigned int size = toWrite.size();
    switch (ch) {
        case '1':
            toWrite = "vt" + toWrite;
            break;
        case '2':
            toWrite = "n," + toWrite;
            break;
        case '3':
            toWrite = "adj" + toWrite;
            break;
        case '4':
            toWrite = "n," + toWrite;
            break;
        case '5':
            toWrite = "fw" + toWrite;
            break;
        case '6':
            toWrite = "prep" + toWrite;
            break;
    }

    if (size == toWrite.size()) {
        return false;
    }

    jeliza::strings vec;

    util::split(toWrite, std::string(";"), vec);
    while (vec.size() < 4) {
        vec.push_back(s("_"));
    }
    add_wortschatz_word(vec);

    jeliza::force_answer_check = false;
    jeliza::force_answer_word = "";
    jeliza::force_answer.set("");

    return true;
}

/*
 * Ein Genus in die Datenbank eintragen, nach erzwungener Antwort
 */
bool jeliza::jeliza_impl::add_genus(char ch, std::string genus) {
    jeliza::string toWrite = jeliza::string(";" + genus);
    unsigned int size = toWrite.size();
    switch (ch) {
        case '1':
            toWrite = "male" + toWrite;
            break;
        case '2':
            toWrite = "female" + toWrite;
            break;
        case '3':
            toWrite = "thing" + toWrite;
            break;
    }

    if (size == toWrite.size()) {
        return false;
    }

    jeliza::strings vec;

    util::split(toWrite, std::string(";"), vec);
    add_wortschatz_genus(vec);

    jeliza::force_answer_genus = "";

    return true;
}

/*
 * Wrapper fuer std::find
 */
jeliza::strings jeliza::findit(shared_ptr<vector<Pair<std::string, jeliza::string> > >* vec, std::string str) {
    vector<Pair<std::string, jeliza::string> >::iterator it = find((*vec)->begin(), (*vec)->end(), Pair<std::string, jeliza::string>(str));
    if (it == (*vec)->end()) {
        return jeliza::strings();
    }

    ++it;
    jeliza::strings ret;
    ret.push_back(it->second);
    return ret;
}

/*
 * Encodiere URLs
 */
jeliza::string url_encode(jeliza::string val)
{
  jeliza::string ret(val);

  ret = ret.replace_nocase_safe(s("\r"), s(""))
           .replace_nocase_safe(s("\n"), s("NEWLINE"))
           .replace_nocase_safe(s(" "), s("%20"))
           .replace_nocase_safe(s("<"), s("%3C"))
           .replace_nocase_safe(s(">"), s("%3E"))
           .replace_nocase_safe(s("#"), s("%23"))
           .replace_nocase_safe(s("%"), s("%25"))
           .replace_nocase_safe(s("{"), s("%7B"))
           .replace_nocase_safe(s("}"), s("%7D"))
           .replace_nocase_safe(s("|"), s("%7C"))
           .replace_nocase_safe(s("\\"), s("%5C"))
           .replace_nocase_safe(s("^"), s("%5E"))
           .replace_nocase_safe(s("~"), s("%7E"))
           .replace_nocase_safe(s("["), s("%5B"))
           .replace_nocase_safe(s("]"), s("%5D"))
           .replace_nocase_safe(s("`"), s("%60"))
           .replace_nocase_safe(s(";"), s("%3B"))
           .replace_nocase_safe(s("/"), s("%2F"))
           .replace_nocase_safe(s("?"), s("%3F"))
           .replace_nocase_safe(s(":"), s("%3A"))
           .replace_nocase_safe(s("@"), s("%40"))
           .replace_nocase_safe(s("="), s("%3D"))
           .replace_nocase_safe(s("&"), s("%26"))
           .replace_nocase_safe(s("$"), s("%24"));

  return ret;
}


/*
 * In XML umwandeln und zurueckgeben
 */
jeliza::string jeliza::db_sentence::toXML() {
    jeliza::string temp;

    std::stringstream sst;
    sst << priority;
    jeliza::string tempprio;
    sst >> tempprio.ref();

    jeliza::string all;

    all += " <fact>\n";
    all += this->printPart (string("prefix>   "), temp, prefix.ref());
    all += this->printPart (string("subject>  "), temp, subject.ref());
    all += this->printPart (string("verb>     "), temp, verb.ref());
    all += this->printPart (string("object>   "), temp, object.ref());
    all += this->printPart (string("suffix>   "), temp, suffix.ref());
    all += this->printPart (string("feeling>  "), temp, feeling.ref());
    all += this->printPart (string("priority> "), temp, tempprio.ref());
    all += this->printPart (string("category> "), temp, category.ref());
    all += this->printPart (string("zusatz>   "), temp, zusatz.ref());
    all += this->printPart (string("quesword> "), temp, quesword.ref());
    for (unsigned int x = 0; x < sub->size(); x++) {
        all += " <sub>\n";
        all += (*sub)[x].toXML();
        all += " </sub>\n";
    }
    all += " </fact>\n\n";

    return all;
}

bool jeliza::db_sentence::operator== (const db_sentence& sent) const {
    if (prefix == sent.prefix
            && subject == sent.subject
            && verb == sent.verb
            && object == sent.object
            && suffix == sent.suffix
            && feeling == sent.feeling
            && zusatz == sent.zusatz
            && quesword == sent.quesword
            ) {
        return true;
    }
    return false;
}


/*
 * In XML umwandeln und in Log(-Datei?) schreiben
 */
void jeliza::db_sentence::toXMLPrint() {
    clogger << toXML().ref() << endl;
}

/*
 * Hilfsfunktion fuer toXML
 */
jeliza::string jeliza::db_sentence::printPart (string str, jeliza::string temp, std::string wert) {
    temp = temp.replace(string("\n"), std::string("\\n"));
    temp = temp.replace(string("\r"), std::string(""));

    return jeliza::string("  <" + str + wert + "  " + "</" + str + "\n");
}

/*
 * Den Satz aufgetrennt menschenlesbar ausgeben
 */
void jeliza::db_sentence::print(std::string begin = "", bool clog = true, bool cou = true) {
    if (clog) {
        clogger << begin << "Subject:   " << subject.ref() << endl;
        clogger << begin << "Verb:      " << verb.ref() << endl;
        clogger << begin << "Object:    " << object.ref() << endl;
        clogger << begin << "Prefix:    " << prefix.ref() << endl;
        clogger << begin << "Suffix:    " << suffix.ref() << endl;
        clogger << begin << "Feeling:   " << feeling.ref() << endl;
        clogger << begin << "Priority:  " << priority << endl;
        clogger << begin << "Category:  " << category.ref() << endl;
        clogger << begin << "Zusatz:    " << zusatz.ref() << endl;
        clogger << begin << "QuesWord:  " << quesword.ref() << endl;
        for (unsigned int x = 0; x < sub->size(); x++) {
            clogger << begin << "Nebensatz " << x+1 << ":" << endl;
            (*sub)[x].print(begin + "  ", true, false);
        }
        clogger << endl;
    }

    if (cou) {
        cout << begin << "Subject:   " << subject.ref() << endl;
        cout << begin << "Verb:      " << verb.ref() << endl;
        cout << begin << "Object:    " << object.ref() << endl;
        cout << begin << "Prefix:    " << prefix.ref() << endl;
        cout << begin << "Suffix:    " << suffix.ref() << endl;
        cout << begin << "Feeling:   " << feeling.ref() << endl;
        cout << begin << "Priority:  " << priority << endl;
        cout << begin << "Category:  " << category.ref() << endl;
        cout << begin << "Zusatz:    " << zusatz.ref() << endl;
        cout << begin << "QuesWord:  " << quesword.ref() << endl;
        for (unsigned int x = 0; x < sub->size(); x++) {
            cout << begin << "Nebensatz " << x+1 << ":" << endl;
            (*sub)[x].print(begin + "  ", false, true);
        }
        cout << endl;
    }
}

/*
 * Leerzeichen am Anfang und Ende von allen SentenceParten entfernen
 */
void jeliza::db_sentence::strip() {
    prefix.strip();
    subject.strip();
    verb.strip();
    object.strip();
    suffix.strip();
    feeling.strip();
    category.strip();
    zusatz.strip();
    quesword.strip();

    prefix.replace(string("\\n"), std::string("\n"));
    subject.replace(string("\\n"), std::string("\n"));
    verb.replace(string("\\n"), std::string("\n"));;
    object.replace(string("\\n"), std::string("\n"));
    suffix.replace(string("\\n"), std::string("\n"));
    feeling.replace(string("\\n"), std::string("\n"));
    category.replace(string("\\n"), std::string("\n"));
    zusatz.replace(string("\\n"), std::string("\n"));
    quesword.replace(string("\\n"), std::string("\n"));

    for (unsigned int x = 0; x < sub->size(); x++) {
        (*sub)[x].strip();
    }
}

jeliza::strings jeliza::db_sentence::genSentences(bool withFix = false) {
    strip();

    jeliza::strings ans;

    if (subject.size() > 2 && verb.size() > 2 && object.size() > 1) {
        ans.push_back(jeliza::string(subject + " " + verb + " " + object));
    } else if (withFix) {
        ans.push_back(jeliza::string(prefix + " " + subject + " " + verb + " " + object + " " + suffix));
    }
    jeliza::string temp;
    (temp = prefix + " " + subject + " " + verb + " " + object).strip();
    if (temp.size() > 1) {
        ans.push_back(temp);
    }
    (temp = subject + " " + verb + " " + object + " " + suffix).strip();
    if (temp.size() > 1) {
        ans.push_back(temp);
    }

    ans.push_back(jeliza::string(prefix + " " + subject + " " + verb + " " + object + " " + suffix));

    return ans;
}

jeliza::strings jeliza::db_sentence::genSentences_all(bool) {
    strip();

    jeliza::strings ans;

    ans.push_back(jeliza::string(prefix + " " + subject + " " + verb + " " + object + " " + suffix));

    return ans;
}

/*
 * Ist es eine Frage?
 */
unsigned int jeliza::jeliza_impl::is_question (jeliza::string ques) {
    ques = " " + ques + " ";
    ques.lower();
    if (ques.contains(jeliza::string(" why "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" when "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" who "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" where "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" what "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" which "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" was "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wie "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wer "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wie "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wo "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wann "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" warum "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" wieso "))) {
        return 1;
    }
    if (ques.contains(jeliza::string(" weshalb "))) {
        return 1;
    }

    if (ques.contains(jeliza::string(" kennst du ")) && ques.contains(jeliza::string("?"))) {
        return 1;
    }

    if (ques.contains(jeliza::string(" kenne du ")) && ques.contains(jeliza::string("?"))) {
        return 1;
    }

    if (ques.contains(jeliza::string("?"))) {
        return 2;
    }

    return 0;
}

jeliza::db_sentence& jeliza::db_sentence::operator= (const jeliza::db_sentence& sent) {
    return set(sent);
}

jeliza::db_sentence::db_sentence (const jeliza::db_sentence& sent) {
    set(sent);
}

jeliza::db_sentence& jeliza::db_sentence::set (const jeliza::db_sentence& sent) {
    subject.set(sent.subject);
    verb.set(sent.verb);
    object.set(sent.object);
    prefix.set(sent.prefix);
    suffix.set(sent.suffix);
    feeling.set(sent.feeling);
    category.set(sent.category);
    quesword.set(sent.quesword);
    zusatz.set(sent.zusatz);
    priority = sent.priority;
    sub.reset(new vector<db_sentence>());

    for (unsigned int x = 0; x < sent.sub->size(); x++) {
        db_sentence dbs = (*sent.sub)[x];
        sub->push_back(dbs);
    }

    return (*this);
}

/*
 * Datenbank aus Datei einlesen
 */
jeliza::DB* jeliza::db_manager::parse (std::string file) {
    ifstream i(file.c_str());

    jeliza::string buffer;
    jeliza::string all;
    while (i) {
        getline (i, buffer.ref());

        if (buffer.contains("<?xml")) {
            continue;
        }

        all += buffer;
        all += "\n";
    }

    jeliza::strings lines;
    util::split(all, std::string("\n\r<>"), lines);

    jeliza::DB* sents = new jeliza::DB;

    for (jeliza::strings::iterator it = lines.begin(); it != lines.end(); it++) {
        it->strip();
    }

    jeliza::db_sentence act_sent;
    jeliza::db_sentence act_sent_vor_sub;
    jeliza::string line;
    bool sub_mode = false;
    int a = 0;
    for (jeliza::strings::iterator it = lines.begin(); it != lines.end(); it++) {
        a++;
        line.ref() = it->ref();

        jeliza::update_prozent(100.0 / lines.size() * a);

        if (line.equal("priority")) {
            std::stringstream sst;
            sst << (*(it + 1)).ref();
            int t;
            sst >> t;
            act_sent.priority = t;
            it += 2;
        }

        else if (line.equal("subject")) {
            act_sent.subject.ref() = (it + 1)->get();

            it += 2;
        }

        else if (line.equal("verb")) {
            act_sent.verb.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("object")) {
            act_sent.object.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("feeling")) {
            act_sent.feeling.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("prefix")) {
            act_sent.prefix.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("suffix")) {
            act_sent.suffix.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("category")) {
            act_sent.category.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("zusatz")) {
            act_sent.zusatz.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("quesword")) {
            act_sent.quesword.ref() = (it + 1)->get();
            it += 2;
        }

        else if (line.equal("/fact")) {
            if (!sub_mode) {
                act_sent.strip();
                if (jeliza::jeliza_impl::is_question(act_sent.genSentences_all(true)[0]) == 0) {
                    sents->push_back(act_sent);
                } else {
                    cout << "Ist Frage: " << act_sent.genSentences_all(true)[0].ref() << endl;
                }
                act_sent = jeliza::db_sentence();
            }
        }

        else if (line.equal("/sub")) {
            act_sent.strip();
            act_sent_vor_sub.sub->push_back(act_sent);

            act_sent = act_sent_vor_sub;
            sub_mode = false;
        }

        else if (line.equal("sub")) {
            act_sent_vor_sub = act_sent;
            act_sent = jeliza::db_sentence();
            sub_mode = true;
        }



    }

    return (sents);
}

void berichtige_wenn_dann(jeliza::db_sentence act, jeliza::db_sentence sub) {
    if (sub.quesword.get_lower().contains("wenn") || sub.quesword.get_lower().contains("falls")) {
        sub.quesword.set("");
    }

    if (act.quesword.get_lower().contains("wenn") || act.quesword.get_lower().contains("falls")) {
        act.quesword.set("");

        jeliza::db_sentence tmp;
        tmp = act;
        act = sub;
        sub = tmp;
    }
}

/*
 * Wrapper fuer
 * Trennt den Satz auf
 */
shared_ptr<jeliza::db_sentence> jeliza::spo_split::get_spo (jeliza::string buffer) {
    buffer.replace(jeliza::string("\n"), jeliza::string("\\n"));

    log("--- Hauptsatz-Suche ---");
    jeliza::db_sentence parts = split_spo(buffer, MAIN_CLAUSE);

    log("--- Nebensatz-Suche ---");
    jeliza::db_sentence sub = split_spo(buffer, SUB_CLAUSE);

    jeliza::db_sentence act_sent = parts;

    berichtige_wenn_dann(act_sent, sub);

    if (sub.verb.size() > 0) {
        act_sent.sub->push_back(sub);
    }
    if (act_sent.subject.size() == 0) {
        act_sent = sub; // Fragesatz!
    }

    shared_ptr<jeliza::db_sentence> act_sent_ret(new jeliza::db_sentence());
    (*act_sent_ret) = act_sent;

    return act_sent_ret;
}


/*
 * Speichert die Datenbank
 */
void jeliza::db_manager::save(std::string file) {
    ofstream o(file.c_str());

    o << get_string() << endl;
    o.close();
}

/*
 * Liefert die Datenbank im XML-Format als std::string zurueck
 */
string jeliza::db_manager::get_string() {
    std::string str;

    str += "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n\n<jdb>\n";

    str += " <feelings>\n  <feeling>happy</happy>\n  <feeling>angry</angry>\n  <feeling>bored</bored>\n  <feeling>scared</scared>\n  <feeling>curiously</curiously>\n  <feeling>normal</normal>\n </feelings>\n";

    for (unsigned int x = 0; x < m_db->size(); ++x) {
        db_sentence s((*m_db)[x]);
        str += s.toXML().toASCII().ref();
        str += "\n";
    }

    str += "</jdb>\n";

    return str;
}


/*
 * Vereinfacht einen Satz, z.B. zum Vergleichen
 */
jeliza::string jeliza::ohne_muell(jeliza::string frage) {
	frage = frage.replace(string("?"), std::string(""));
	frage = frage.replace(string("!"), std::string(""));
	frage = frage.replace(string("."), std::string(""));
	frage = frage.replace(string(","), std::string(""));
	frage = frage.replace(string(";"), std::string(""));
	frage = frage.replace(string("+"), std::string(" ztrgftredrefd "));
	frage = frage.replace(string("ztrgftredrefd"), std::string("+"));
	frage = frage.replace(string("*"), std::string(" ztrgftredrefd "));
	frage = frage.replace(string("ztrgftredrefd"), std::string("*"));
	frage = frage.replace(string("/"), std::string(" ztrgftredrefd "));
	frage = frage.replace(string("ztrgftredrefd"), std::string("/"));
	frage = frage.replace(string("  "), std::string(" "));
	frage = " " + frage + " ";

	vector<std::string> unuseful_words;
	unuseful_words.push_back("der");
	unuseful_words.push_back("die");
	unuseful_words.push_back("das");
	unuseful_words.push_back("dem");
	unuseful_words.push_back("des");
	unuseful_words.push_back("dessen");
	unuseful_words.push_back("was");
	unuseful_words.push_back("wer");
	unuseful_words.push_back("wie");
	unuseful_words.push_back("wo");
	unuseful_words.push_back("wann");
	unuseful_words.push_back("von");
	unuseful_words.push_back("vom");
	unuseful_words.push_back("in");
	unuseful_words.push_back("auf");
	unuseful_words.push_back("unter");
	unuseful_words.push_back("neben");
	unuseful_words.push_back("zwischen");
	unuseful_words.push_back("denn");
	unuseful_words.push_back("bloß");
	unuseful_words.push_back("bloss");
	unuseful_words.push_back("stimmts");
	unuseful_words.push_back("stimmt's");
	unuseful_words.push_back("hmm");
	unuseful_words.push_back("hm");
	unuseful_words.push_back("hmmm");
	unuseful_words.push_back("hmmmm");
	unuseful_words.push_back("sonst");
	unuseful_words.push_back("ansonsten");
	unuseful_words.push_back("noch");

	unuseful_words.push_back("fast");
	unuseful_words.push_back("bald");
	unuseful_words.push_back("sehr");
	unuseful_words.push_back("na");
	unuseful_words.push_back("bisschen");
	unuseful_words.push_back("bischen");
	unuseful_words.push_back("hald");
	unuseful_words.push_back("halt");
	unuseful_words.push_back("eben");
	unuseful_words.push_back("oder");
	unuseful_words.push_back("und");

	unuseful_words.push_back("of");
	unuseful_words.push_back("from");
	unuseful_words.push_back("by");
	unuseful_words.push_back("between");
	unuseful_words.push_back("what");
	unuseful_words.push_back("when");

	unuseful_words.push_back("ein");
	unuseful_words.push_back("eine");
	unuseful_words.push_back("einer");
	unuseful_words.push_back("eines");
	unuseful_words.push_back("einem");

	unuseful_words.push_back("fr");
	unuseful_words.push_back("für");
	unuseful_words.push_back("fuer");

	for (unsigned int x = 0; x < unuseful_words.size(); x++) {
		jeliza::string unuseful_word(" " + unuseful_words[x] + " ");

		jeliza::string better_frage(frage.replace(unuseful_word, jeliza::string(" ")));

		if (better_frage.get_strip().size() > 0) {
			frage.set(better_frage);
		}
	}

	return frage.get_strip();
}

/*
 * Ist "ch" in "all"?
 */
bool is_in (string all, char ch) {
    std::string::iterator iter = find(all.begin(), all.end(), ch);

    if (iter == all.end()) {
        return false;
    }
    return true;
}

const std::string vokale("aeiouAEIOU");
const std::string konsonanten("bcdfghjklmnpqrstvwxyzBCDFGHJKLMNPQRSTVWXYZ");
const jeliza::string prefixe("ab an auf aus be bei dar ein er ent ge miss hin nach nieder ueber ber um unter ver vor weg zer zu");

/*
 * Entfernt Praefixe
 */
jeliza::string jeliza::spo_split::entferne_prefixe(jeliza::string s1) {
    if (s1.size() < 6) {
        return s1;
    }

    jeliza::strings v_prefixe;
    jeliza::string prefixe_tmp(prefixe);
    util::split(prefixe_tmp, std::string(" "), v_prefixe);

    for (int x = 0; x < 3; x++) {
        for (unsigned int y = 0; y < v_prefixe.size(); y++) {
            if (s1.get_lower().startswith(v_prefixe[y])) {
                s1.replace_first_nocase(v_prefixe[y].get_lower(), jeliza::string(""));
            }
        }
    }

    return s1;
}

jeliza::spo_split& jeliza::spo_split::get() {
    return instance;
}

/*
 * Ermittelt den Wortstamm
 */
jeliza::string jeliza::spo_split::ermittle_wortstamm(jeliza::string s1) {
    jeliza::string stamm;

    jeliza::string orig_s1(s1);
    s1 = entferne_prefixe(s1);

    int i = 0;
    if (is_in(konsonanten, s1[0])) {
        while (is_in(konsonanten, s1[i])) {
            stamm.ref() += s1[i];
            i++;
        }
    }
    while (is_in(vokale, s1[i])) {
        stamm.ref() += s1[i];
        i++;
    }
    while ((s1.size() - i > 2 || s1.size() < 4 || s1[i] == s1[i-1]) && is_in(konsonanten, s1[i])) {
        stamm.ref() += s1[i];
        i++;
    }

    return stamm;
}

/*
 * Gleicher Wortstamm?
 */
bool jeliza::spo_split::hat_gleichen_wortstamm(jeliza::string s1, jeliza::string s2, jeliza::string stamm1, jeliza::string stamm2) {
    if (s1 == s2) {
        return true;
    }

    if (s1 == stamm1) {
        return false;
    }
    if (s2 == stamm2) {
        return false;
    }

    if (stamm1 == stamm2) {
        return true;
    }

    s1.strip();
    s1.replace("_", " ");

    s2.strip();
    s2.replace("_", " ");

    if (s1.size() < 1 && s2.size() > 0) {
        return false;
    }
    if (s2.size() < 1 && s1.size() > 0) {
        return false;
    }

    if (s1 == s2) {
        return true;
    }

    bool valid_s1 = s1.get_lower() == s1;
    bool valid_s2 = s2.get_lower() == s2;
    if (valid_s1 != valid_s2) {
        return false;
    }

    if (s1.size() > 18) {
        return false;
    }
    if (s2.size() > 18) {
        return false;
    }

    return false;
}

/*
 * Sind sich die std::strings ziemlich aehnlich?
 */
bool jeliza::is_similar(jeliza::string s1, jeliza::string s2) {
    if (s1 == s2) {
        return true;
    }

    s1.lower();
    s1.strip();
    s1.replace("_", " ");

    s2.lower();
    s2.strip();
    s2.replace("_", " ");

    if (s1.size() < 1 && s2.size() > 0) {
        return false;
    }
    if (s2.size() < 1 && s1.size() > 0) {
        return false;
    }

    if (s1 == s2) {
        return true;
    }
    if (s1.contains(s2)) {
        return true;
    }
    if (s2.contains(s1)) {
        return true;
    }
    return false;
}


class number_format
{
    std::string What;

    public:
    number_format() : What("Typumwandlung misslungen")
    {
    }

    number_format(std::string what) : What(what)
    {
    }

    std::string what(void)
    {
      return What;
    }
};

long string_to_long(std::string str)
{
  if (str.empty())
    throw number_format();
  char pos = 0;
  if (str[0] == '-')
    ++pos;
  if (str[pos] < 48 || str[pos] > 57)
    throw number_format();
  return std::strtol(str.c_str(), NULL, 10);
}

/*
 * Bestimmt die Wordtype, wobei Pronomen und Adjektive als Nomen gewertet werden
 */
jeliza::Wordtype jeliza::spo_split::bestimmeWordtypeWrapper(jeliza::string word2, const bool atBeginningOfSentence) {
    Wordtype ret = bestimmeWordtype(word2, atBeginningOfSentence);
    if (ret == PP) {
        ret = NOUN;
    }
    return ret;
}

/*
 * Bestimmt die Wordtype
 */
jeliza::Wordtype jeliza::spo_split::bestimmeWordtype(jeliza::string word2, const bool atBeginningOfSentence) {
	word2 = word2.replace("{", "");
	word2 = word2.replace("}", "");
    if (word2.get_strip().size() == 0) {
        return NO_WORD_TYPE;
    }
    std::string word = word2.ref();
    if (word == "?" || word == "." || word == "," || word == ";") {
        return NO_WORD_TYPE;
    }

    word2 = word2.replace(".", "").replace("?", "").replace("!", "").replace(",", "").replace(";", "").replace("\"", "").replace("'", "");
    word = word2.ref();

    std::string str;
    std::string word_low = word2.get_lower().ref();
    if (word_low == "und" || word_low == "oder" || word_low == "or" || word_low == "and") {
        return LINKING;
    }

    std::string word_substrd = word.substr(0, word.size() - 3);

    jeliza::string prefixe_entfernt_word = entferne_prefixe(word2);
    jeliza::string prefixe_entfernt_word_low = entferne_prefixe(word2.get_lower());

    ifstream iVerbs(("lang_" + jeliza::get_lang_str() + "/wsh/verbs.wsh").ref().c_str());
    vector<std::string> vVerbs;
    while (iVerbs) {
        iVerbs >> str;
        vVerbs.push_back(str);
    }

    if (find(vVerbs.begin(), vVerbs.end(), word) != vVerbs.end()) {
        return VERB;
    }

    // Artikel
    ifstream iArts(("lang_" + jeliza::get_lang_str() + "/wsh/arts.wsh").ref().c_str());
    vector<std::string> vArts;
    while (iArts) {
        iArts >> str;
        vArts.push_back(str);
    }

    if (find(vArts.begin(), vArts.end(), word_low) != vArts.end()) {
        return ART;
    }

    // Pronomen
    ifstream iPP(("lang_" + jeliza::get_lang_str() + "/wsh/pp.wsh").ref().c_str());
    vector<std::string> vPP;
    while (iPP) {
        iPP >> str;
        vPP.push_back(str);
    }

    if (find(vPP.begin(), vPP.end(), word_low) != vPP.end()) {
        return PP;
    }

    // Praepositionen
    ifstream iPreps(("lang_" + jeliza::get_lang_str() + "/wsh/preps.wsh").ref().c_str());
    vector<std::string> vPreps;
    while (iPreps) {
        iPreps >> str;
        vPreps.push_back(str);
    }

    if (find(vPreps.begin(), vPreps.end(), word_low) != vPreps.end()) {
        return PREP;
    }

    jeliza::string vec_0;
    if (!atBeginningOfSentence) {
        if (jeliza::findit(&Wordtype_equal, word2.ref()).size() > 0) {
            vec_0.set(jeliza::findit(&Wordtype_equal, word2.ref())[0]);
        }
//        else if (jeliza::findit(&Wordtype_ohne_prefix, prefixe_entfernt_word.ref()).size() > 0) {
//            vec_0.set(jeliza::findit(&Wordtype_ohne_prefix, prefixe_entfernt_word.ref())[0]);
//        }
    }
    else {
        if (jeliza::findit(&Wordtype_equal_low, word_low).size() > 0) {
            vec_0.set(jeliza::findit(&Wordtype_equal_low, word_low)[0]);
        }
        else if (jeliza::findit(&Wordtype_equal, word2.ref()).size() > 0) {
            vec_0.set(jeliza::findit(&Wordtype_equal, word2.ref())[0]);
        }
//        else if (jeliza::findit(&Wordtype_ohne_prefix, prefixe_entfernt_word.ref()).size() > 0) {
//            vec_0.set(jeliza::findit(&Wordtype_ohne_prefix, prefixe_entfernt_word.ref())[0]);
//        }
    }
    if (vec_0.size() > 0) {
        if (vec_0 == "prep") {
            return PREP;
        }
        if (vec_0 == "fw") {
            return QUESTIONWORD;
        }

        if (vec_0 == "verb") {
            return VERB;
        }
        if (vec_0 == "vt") {
            return VERB;
        }
        if (vec_0 == "vi") {
            return VERB;
        }
        if (vec_0 == "n") {
            return NOUN;
        }
        if (vec_0 == "f,m") {
            return NOUN;
        }
        if (vec_0 == "f,n") {
            return NOUN;
        }
        if (vec_0 == "f,n,m") {
            return NOUN;
        }
        if (vec_0 == "f,m,n") {
            return NOUN;
        }
        if (vec_0 == "m,f") {
            return NOUN;
        }
        if (vec_0 == "m,n") {
            return NOUN;
        }
        if (vec_0 == "m,f,n") {
            return NOUN;
        }
        if (vec_0 == "m,n,f") {
            return NOUN;
        }
        if (vec_0 == "m,") {
            return NOUN;
        }
        if (vec_0 == "f,") {
            return NOUN;
        }
        if (vec_0 == "n,") {
            return NOUN;
        }
        if (vec_0 == "n,pl") {
            return NOUN;
        }

        if (vec_0 == "adj") {
            return ADJ;
        }
        if (vec_0 == "adv") {
            return ADJ;
        }
        if (vec_0 == "pron") {
            return PP;
        }
        if (vec_0 == "ppron") {
            return NOUN;
        }
        if (vec_0 == "nothing") {
            return UNIMPORTANT;
        }
    }

    try {
        string_to_long(word_low);
        return NOUN;
    }
    catch (number_format &ex) {
//        std::cerr << ex.what() << std::endl;
    }

    while (not(jeliza::next_word_type == "-1") || jeliza::next_word_type == "0") {
            jeliza::msleep(500);
    }

    {
        jeliza::next_word_type = "0";

        unsigned int a = 5;
        while (jeliza::next_word_type[0] == '0') {
            if (a % 5 == 0) {
                jeliza::comm_send("GET_WORD_TYPE:" + word2.ref());
            }

            ++a;
            jeliza::msleep(500);
            cout << "Waiting... " << jeliza::next_word_type << endl;
            if (jeliza::fast_exit) {
            	return NO_WORD_TYPE;
            }
        }
        cout << "500" << endl;

        if (jeliza::next_word_type.size() > 0) {
            jeliza::force_answer_word = word2.ref();
            jeliza_impl_instance.add_word(jeliza::next_word_type[0]);
        }
        else {
            cout << "Bug gefunden! Nichts eingegeben in GUI oder falsch gesendet!" << endl;
        }
        cout << "501" << endl;

        jeliza::next_word_type = "-1";
        cout << "502" << endl;
        return bestimmeWordtype(word2, atBeginningOfSentence);
    }


    return NO_WORD_TYPE;
}

jeliza::Genus jeliza::spo_split::bestimmeGenus(jeliza::string word2) {
	word2 = word2.replace("{", "");
	word2 = word2.replace("}", "");
    if (word2.get_strip().size() == 0) {
        return NO_GENUS;
    }
    std::string word = word2.ref();
    if (word == "?" || word == "." || word == "," || word == ";") {
        return NO_GENUS;
    }

    word2 = word2.replace(".", "").replace("?", "").replace("!", "").replace(",", "").replace(";", "").replace("\"", "").replace("'", "");
    word = word2.ref();

    std::string str;
    std::string word_low = word2.get_lower().ref();
    if (word_low == "und" || word_low == "oder" || word_low == "or" || word_low == "and") {
        return NO_GENUS;
    }


    std::string genus = (*jeliza::spo_split::get().genus_index)[word];
    if (genus.size()) {
    	if (genus == "male") cout << word << " MALE" << endl;
    	if (genus == "female") cout << word << " FEMALE" << endl;
    	if (genus == "thing") cout << word << " THING" << endl;
    	if (genus == "male") return MALE;
    	if (genus == "female") return FEMALE;
    	if (genus == "thing") return THING;
    	cout << "Unbekannter Genus: " << genus << endl;
    }
	cout << "Unbekannter Genus: " << genus << endl;


    while (not(jeliza::next_genus == "-1") || jeliza::next_genus == "0") {
            jeliza::msleep(500);
    }

    {
        jeliza::next_genus = "0";

        unsigned int a = 5;
        while (jeliza::next_genus[0] == '0') {
            if (a % 5 == 0) {
                jeliza::comm_send("GET_GENUS:" + word2.ref());
            }

            ++a;
            jeliza::msleep(500);
            cout << "Waiting... " << jeliza::next_genus << endl;
            if (jeliza::fast_exit) {
            	return NO_GENUS;
            }
        }
        cout << "500" << endl;

        if (jeliza::next_genus.size() > 0) {
            jeliza_impl_instance.add_genus(jeliza::next_genus[0], word);
        }
        else {
            cout << "Bug gefunden! Nichts eingegeben in GUI oder falsch gesendet!" << endl;
        }
        cout << "501" << endl;

        jeliza::next_genus = "-1";
        cout << "502" << endl;
        return bestimmeGenus(word2);
    }


    return NO_GENUS;
}



jeliza::worddef* next_word_if(vector<jeliza::worddef>& woerter, jeliza::Wordtype soll_wa, unsigned int index) {
    for (unsigned int x = index; x < woerter.size(); x++) {
        if (woerter[x].second == soll_wa) {
            return (&woerter[x]);
        }
    }
    return NULL;
}

unsigned int next_word_index_if(vector<jeliza::worddef>& woerter, jeliza::Wordtype soll_wa, unsigned int index) {
    for (unsigned int x = index; x < woerter.size(); x++) {
        if (woerter[x].second == soll_wa) {
            return x;
        }
    }
    return -1;
}

bool pred_lexicographical_compare (const jeliza::string& lhs, const jeliza::string& rhs) {
    return std::lexicographical_compare(lhs.get_lower().begin(), lhs.size()>10 ?
            lhs.get_lower().begin()+10 : lhs.get_lower().end(), rhs.get_lower().begin(), rhs.size()>10 ? rhs.get_lower().begin()+10 :
            rhs.get_lower().end());
}

/*
 * Satzauftrennungsdatenbank initialisieren
 */
void jeliza::spo_split::init_sentences_splitted() {
    {
        sentences_splitted = map<vector<jeliza::Wordtype>, vector<pair<jeliza::Wordtype, pair<jeliza::SentencePart, jeliza::SentencePart> > > >();

        ifstream i("temp/sentences.tmp");
        jeliza::string line;
        unsigned long a = 0;
        vector<jeliza::string> parts;
        while (i) {
            getline(i, line.ref());

            ++a;
            if (a % 100 == 0) {
                jeliza::update_prozent(100.0 / 80000.0 * a);
            }

            vector<vector<pair<Wordtype, pair<SentencePart, SentencePart> > > > musterLines;
            vector<vector<Wordtype> > keys;

            vector<pair<Wordtype, pair<SentencePart, SentencePart> > > musterLine;
            vector<Wordtype> key;
            musterLines.push_back(musterLine);
            keys.push_back(key);

            util::split(line, std::string(" "), parts);
            Wordtype i = NO_WORD_TYPE;
            SentencePart s = NO_CLAUSE;
            SentencePart os = NO_CLAUSE;
            for (unsigned int x = 0; x < parts.size(); ++x) {
                parts[x].strip();
                if (parts[x].contains("'''")) {
                    if (parts[x].contains("SUBJ")) {
                        s = SUBJ;
                    }
                    if (parts[x].contains("OBJ")) {
                        s = OBJ;
                    }
                    if (parts[x].contains("PRE")) {
                        s = PRAE;
                    }
                    if (parts[x].contains("ZUS")) {
                        s = PROPERTIES;
                    }

                    if (parts[x].contains("HS")) {
                        os = MAIN_CLAUSE;
                    }
                    if (parts[x].contains("NS")) {
                        os = SUB_CLAUSE;
                    }
                    if (parts[x].contains("FS")) {
                        os = MAIN_CLAUSE;
                    }

                    continue;
                }

                if (parts[x].contains("ART")) i = ART;
                else if (parts[x].contains("ADJ")) i = ADJ;
                else if (parts[x].contains("N")) i = NOUN;
                else if (parts[x].contains("SPO")) i = SPO;
                else if (parts[x].contains("VERB")) i = VERB;
                else if (parts[x].contains("QW")) i = QUESTIONWORD;
                else if (parts[x].contains("PRPO")) i = PREP;
                else if (parts[x].contains("KOMMA")) i = KOMMA;
                else if (parts[x].contains("LINKING")) i = LINKING;

                for (unsigned int y = 0; y < musterLines.size(); ++y) {
                    musterLines[y].push_back(make_pair(i, make_pair(s, os)));
                    keys[y].push_back(i);
                }
            }

            for (unsigned int x = 0; x < musterLines.size(); ++x) {
                sentences_splitted[keys[x]] = musterLines[x];
            }
        }
    }
}

/*
 * Trennt einen Satz auf
 */
jeliza::db_sentence jeliza::spo_split::split_spo (jeliza::string s, SentencePart SentencePart) {
	s = s.replace("{", "");
	s = s.replace("}", "");
	s = s.replace("  ", " ");
	s = s.replace("  ", " ");
	s = s.replace("  ", " ");
	s = s.replace("  ", " ");

    jeliza::strings words;
    /*
     * Diese Zeichen werden entfernt
     */
    std::string findd = ".()/-:;!?\n\r ";
    /*
     * Das Komma wird zu einem ganzen Wort
     */
    s = s.replace_nocase_safe(jeliza::string(", "), jeliza::string(","));
    s = s.replace_nocase_safe(jeliza::string(" ,"), jeliza::string(","));
    s = s.replace_nocase_safe(jeliza::string(","), jeliza::string(" , "));
    s = s.replace("  ", " ");
    /*
     * Zeichen von oben entfernen
     */
    util::split(s, findd, words);
    util::join(s, std::string(" "), words);

    {
        /*
         * Adverbiale Bestimmungen und alles andere Trennen
         */

        jeliza::strings words;
        util::split(s, std::string(" "), words);
        vector<std::pair<worddef, jeliza::string> > woerter;
        jeliza::string adverbiale_bestimmungen;
        jeliza::string subj_hinten_dran;
        bool adverbiale_bestimmung = false;
        jeliza::string lastWord;
        Wordtype lastWa = WORDTYPE_UNKNOWN;
        Wordtype firstWa = WORDTYPE_UNKNOWN;
        jeliza::string advb_bestim;
        for (unsigned int x = 0; x < words.size(); ++x) {
            jeliza::string word(words[x]);
            jeliza::string word_next;
            unsigned int d;
            for (d = x; d < words.size(); ++d) {
                jeliza::string word(words[d]);
                Wordtype wa = (word == ",") ? KOMMA : bestimmeWordtypeWrapper(word, d == 0);
                if (wa == PP) wa = NOUN;
                if (wa == NOUN) {
                	word_next.set(words[d]);
                	break;
                }
            }
            jeliza::string word_tmp(words[x]);
            word.strip();
            word_tmp.strip();
            Wordtype wa = (word == ",") ? KOMMA : bestimmeWordtypeWrapper(word, x == 0);
            Wordtype wa_echt = (word == ",") ? KOMMA : bestimmeWordtype(word, x == 0);


            if (wa_echt == UNIMPORTANT) {
                continue;
            }

            if (firstWa == WORDTYPE_UNKNOWN) {
                firstWa = wa;
            }
            if (wa == PP) wa = NOUN;

            if (wa == ART) {
                word_tmp += " - Artikel";
            }
            if (wa == ADJ) {
                word_tmp += " - Adjektiv";
            }
            if (wa == NOUN) {
                word_tmp += " - NOUN";
            }
            if (wa == VERB) {
                word_tmp += " - Verb";
            }
            if (wa == QUESTIONWORD) {
                word_tmp += " - Fragewort";
            }
            if (wa == PREP) {
                word_tmp += " - Praeposition";
            }
            if (wa == SPO) {
                word_tmp += " - Subjekt/Verb/Objekt";
            }
            if (wa == KOMMA) {
                word_tmp += " - ein Komma";
            }


            if (wa == lastWa && wa == NOUN && adverbiale_bestimmung) {
                adverbiale_bestimmung = false;
            }

            if (wa == PREP) {
                adverbiale_bestimmung = true;
            }
            if (wa == VERB) {
                adverbiale_bestimmung = false;
            }

            if (wa == ART && jeliza::get_lang() == GERMAN) {
            	cout << "word_next:" << word_next.ref() << endl;
            	Genus g = bestimmeGenus(word_next);

            	if (g == MALE) {
            		if (word.get_lower().ref() != "der"
            			&& word.get_lower().ref() != "ein"
                		&& word.get_lower().ref() != "einen"
                        && word.get_lower().ref() != "mein"
                        && word.get_lower().ref() != "meines"
                        && word.get_lower().ref() != "dein"
                        && word.get_lower().ref() != "deines"
                        && word.get_lower().ref() != "sein"
                        && word.get_lower().ref() != "seines"
                        && word.get_lower().ref() != "unser"
                        && word.get_lower().ref() != "unseres"
                        && word.get_lower().ref() != "unsers"
                        && word.get_lower().ref() != "euer"
                        && word.get_lower().ref() != "eures"
                        && word.get_lower().ref() != "eueres"
                        && word.get_lower().ref() != "euers"
                        && word.get_lower().ref() != "ihr"
                        && word.get_lower().ref() != "ihres"
            		) {
                    	adverbiale_bestimmung = true;
            		}
            	}
            	if (g == FEMALE) {
            		if (word.get_lower().ref() != "die"
            			&& word.get_lower().ref() != "eine"
            			&& word.get_lower().ref() != "einer"
                            && word.get_lower().ref() != "meine"
                            && word.get_lower().ref() != "meiner"
                            && word.get_lower().ref() != "deine"
                            && word.get_lower().ref() != "deiner"
                            && word.get_lower().ref() != "seine"
                            && word.get_lower().ref() != "seiner"
                            && word.get_lower().ref() != "unsere"
                            && word.get_lower().ref() != "unserer"
                            && word.get_lower().ref() != "unserer"
                            && word.get_lower().ref() != "euere"
                            && word.get_lower().ref() != "eurer"
                            && word.get_lower().ref() != "euerer"
                            && word.get_lower().ref() != "euere"
                            && word.get_lower().ref() != "ihre"
                            && word.get_lower().ref() != "ihrer"
            		) {
                    	adverbiale_bestimmung = true;
            		}
            	}
            	if (g == THING) {
            		if (word.get_lower().ref() != "das"
            			&& word.get_lower().ref() != "eines"
            			&& word.get_lower().ref() != "ein"
                            && word.get_lower().ref() != "mein"
                            && word.get_lower().ref() != "meines"
                            && word.get_lower().ref() != "dein"
                            && word.get_lower().ref() != "deines"
                            && word.get_lower().ref() != "sein"
                            && word.get_lower().ref() != "seines"
                            && word.get_lower().ref() != "unser"
                            && word.get_lower().ref() != "unseres"
                            && word.get_lower().ref() != "unsers"
                            && word.get_lower().ref() != "euer"
                            && word.get_lower().ref() != "eures"
                            && word.get_lower().ref() != "eueres"
                            && word.get_lower().ref() != "euers"
                            && word.get_lower().ref() != "ihr"
                            && word.get_lower().ref() != "ihres"
            		) {
                    	adverbiale_bestimmung = true;
            		}
            	}
            }

            bool ist_einzelnes_adjektiv = false;
            if (wa_echt == ADJ) {
                bool hat_kein_nomen_dahinter = false;
                if (x + 1 >= words.size()) {
                    hat_kein_nomen_dahinter = true;
                }
                if (x + 1 < words.size()) {
                    Wordtype wa__ = bestimmeWordtype(words[x+1], false);
                    if (wa__ != NOUN && wa__ != PP) {
                        hat_kein_nomen_dahinter = true;
                    }
                }
                if (hat_kein_nomen_dahinter) {
                    ist_einzelnes_adjektiv = true;
                }
            }
            bool vor_fragewort = false;
            if (x + 1 < words.size()) {
                Wordtype wa__ = bestimmeWordtype(words[x+1], false);
                if (wa__ == QUESTIONWORD) {
                    vor_fragewort = true;
                    adverbiale_bestimmung = false;
                }
            }
            bool nach_fragewort = false;
            if (x - 1 < words.size() && (int)(x) - 1 >= 0) {
                Wordtype wa__ = bestimmeWordtype(words[x-1], false);
                if (wa__ == QUESTIONWORD &&
                		(
                				not(words[x-1].get_strip().get_lower() == "wenn")
                				or not(words[x-1].get_strip().get_lower() == "falls")
                				or not(words[x-1].get_strip().get_lower() == "sobald")
                				or not(words[x-1].get_strip().get_lower() == "if")
                				or not(words[x-1].get_strip().get_lower() == "when")

                		)) {
                    nach_fragewort = true;
                    adverbiale_bestimmung = false;
                }
            }
            bool verb_in_sicht = false;
            if (x < words.size()) {
                Wordtype wa__ = bestimmeWordtype(words[x], false);
                if (wa__ == VERB) {
                    verb_in_sicht = true;
                }
            }
            if (x + 1 < words.size()) {
                Wordtype wa__ = bestimmeWordtype(words[x+1], false);
                if (wa__ == VERB) {
                    verb_in_sicht = true;
                }
            }
            if (x + 2 < words.size()) {
                Wordtype wa__ = bestimmeWordtype(words[x+2], false);
                if (wa__ == VERB) {
                    verb_in_sicht = true;
                }
            }
            if (x + 3 < words.size()) {
                Wordtype wa__ = bestimmeWordtype(words[x+3], false);
                if (wa__ == VERB) {
                    verb_in_sicht = true;
                }
            }
/*
 *             if (x+3 >= words.size()) {
 * 		          verb_in_sicht = false;
 *             }
 */

//            if ((adverbiale_bestimmung || ist_einzelnes_adjektiv) && !vor_fragewort && !nach_fragewort) {
            if ((vor_fragewort || nach_fragewort) && (wa == ART || wa == NOUN || wa == ADJ || wa == PREP) && verb_in_sicht && ((( (int)(x)-1 >= 0) ? (words[x-1]) : jeliza::string()) != jeliza::string("dass"))) {
                advb_bestim += " ";
                advb_bestim += word;
                advb_bestim.strip();
            }
            else if (adverbiale_bestimmung) {
            	advb_bestim += " ";
                advb_bestim += word;
                advb_bestim.strip();
            }
            else if (ist_einzelnes_adjektiv) {
                advb_bestim += " ";
                advb_bestim += word;
                advb_bestim.strip();
            }
            else {
                if (woerter.size()) {
                	woerter[woerter.size() - 1].second += advb_bestim;
                    woerter.push_back(std::pair<worddef, jeliza::string>(worddef(word, wa), jeliza::string()));
                }
                else {
                    woerter.push_back(std::pair<worddef, jeliza::string>(worddef(word, wa), advb_bestim));
                }
                if (advb_bestim.size() != 0) {
                    advb_bestim.set("");
                }
                adverbiale_bestimmung = false;
            }

            cout << adverbiale_bestimmung << ist_einzelnes_adjektiv << vor_fragewort << nach_fragewort << verb_in_sicht
                 << " " << word_tmp.ref() << wa << endl;

            lastWa = wa;
            lastWord.set(word);
        }

        if (advb_bestim.size() != 0) {
            if (woerter.size() > 0) {
                woerter[woerter.size() - 1].second += " ";
                woerter[woerter.size() - 1].second += advb_bestim;
                woerter[woerter.size() - 1].second.strip();
                advb_bestim.set("");
            }
        }
        else {
            woerter.push_back(std::pair<worddef, jeliza::string>(worddef(jeliza::string(), lastWa), advb_bestim));
        }


        /*
         * Ueberpruefen, ob die Trenndatenbank zum trenne verwendet werden kann
         *
         * Vorbereitung:
         */
        log("Pruefen...");
        vector<pair<Wordtype, pair<jeliza::SentencePart, jeliza::SentencePart> > > musterLine;
        bool passt = false;

        vector<Wordtype> key;
        map<unsigned int, unsigned int> index_lang_zu_kurz;
        for (unsigned int x = 0; x < woerter.size(); ++x) {
            Wordtype wa = woerter[x].first.second;
            if (wa == ART) wa = SPO;
            if (wa == NOUN) wa = SPO;
            if (wa == VERB) wa = SPO;
            if (wa == ADJ) wa = SPO;
            if (wa == LINKING) wa = SPO;
            key.push_back(wa);
            index_lang_zu_kurz[x] = x;

            jeliza::string word_tmp;
            if (wa == PP) wa = NOUN;

            if (wa == ART) {
                word_tmp += " - article";
            }
            if (wa == ADJ) {
                word_tmp += " - adjective";
            }
            if (wa == NOUN) {
                word_tmp += " - noun";
            }
            if (wa == VERB) {
                word_tmp += " - verb";
            }
            if (wa == QUESTIONWORD) {
                word_tmp += " - question word";
            }
            if (wa == PREP) {
                word_tmp += " - preposition";
            }
            if (wa == SPO) {
                word_tmp += " - subject/verb/object";
            }
            if (wa == KOMMA) {
                word_tmp += " - komma";
            }
            log(word_tmp.ref());
        }

        /*
         * Passt es?
         */
        musterLine = jeliza::spo_split::get().sentences_splitted[key];
        if (musterLine.size() > 0) {
            log("PASST!");
            passt = true;
        }

        /*
         * Alternativer und primitiver Auftrenner
         */
        if (!passt) {
            log("PASST NICHT!");

            jeliza::db_sentence ret;

            return ret;
        }

        /*
         * Datenbank konnte verwendet werden
         */
        jeliza::string subj;
        jeliza::string prae;
        jeliza::string obj;
        jeliza::string wie_frage_objekt;
        jeliza::string fragewort;
        jeliza::string adv_bestm;
        unsigned int hatte_schon_subjekt = 0; // 0 == Subjekt, 1 == Objekt
        cout << endl << endl << endl << endl;
        for (unsigned int x = 0; x < woerter.size(); ++x) {
            jeliza::string w(woerter[x].first.first);
            Wordtype wa1 = woerter[x].first.second;

            /*
             * Nur den relevanten Satteil betrachten
             */
            cout << musterLine[index_lang_zu_kurz[x]].second.second << musterLine[x].second.second << w.ref() << endl;
            if (musterLine[index_lang_zu_kurz[x]].second.second != SentencePart) {
                continue;
            }
            if (wa1 == QUESTIONWORD && SentencePart != SUB_CLAUSE) {
                break;
            }

            /*
             * Pronomen sind hier Nomen
             */
            if (wa1 == PP) wa1 = NOUN;

            jeliza::strings words;
            util::split(w, std::string(" "), words);
            for (unsigned int y = 0; y < words.size(); ++y) {
                jeliza::string word(words[y]);

                if (word == ",") {
                    continue;
                }

                if (wa1 == PREP) {
                    wie_frage_objekt += word + " ";
                	adverbiale_bestimmungen += " ";
                	adverbiale_bestimmungen += woerter[x].second;
                	adverbiale_bestimmungen.strip();
                }
                if (wa1 == QUESTIONWORD) {
                    fragewort += word;
                    fragewort += " ";
                    hatte_schon_subjekt = 0;
                    wie_frage_objekt += subj + " " + obj + " ";
                    subj.set("");
                    obj.set("");
                	adverbiale_bestimmungen += " ";
                	adverbiale_bestimmungen += woerter[x].second;
                	adverbiale_bestimmungen.strip();
                }
                else if (wa1 == VERB) {
                    prae += word;
                    prae += " ";

                    subj.strip();
                    if (subj.size() > 0) {
                        hatte_schon_subjekt++;
                    }
                	adverbiale_bestimmungen += " ";
                	adverbiale_bestimmungen += woerter[x].second;
                	adverbiale_bestimmungen.strip();
                }
                else if (musterLine[x].second.second == PROPERTIES) {
                    wie_frage_objekt += word + " ";
                }
                else {
                    Wordtype wa_ = bestimmeWordtype(word, x == 0);
                    Wordtype wa_next;
                    if (y+1 < words.size()) {
                    	wa_next = bestimmeWordtype(words[y+1], x == 0);
                    } else if (x+1 < woerter.size()) {
                    	jeliza::string w2(woerter[x+1].first.first);
                    	jeliza::strings words_2;
                        util::split(w2, std::string(" "), words_2);
                        if (words_2.size()) {
                        	wa_next = bestimmeWordtype(words_2[0], x+1 == 0);
                        }
                    }

                    if (wa_ == LINKING) {
                        hatte_schon_subjekt--;
                    }

                    adv_bestm += " ";
                    adv_bestm += woerter[x].second;
                    adv_bestm.strip();

                    if (!hatte_schon_subjekt) {
                    	subj += word;
                        subj += " ";

                        if (wa1 == NOUN && (wa_next != LINKING)) {
                        	subj += "{";
                        	subj += adv_bestm;
                        	subj += "} ";
                        	adv_bestm.set("");
                        }
                    }

                    else {
                        obj += word;
                        obj += " ";

                        if (wa1 == NOUN && (wa_next != LINKING)) {
                        	obj += "{";
                        	obj += adv_bestm;
                        	obj += "} ";
                        	adv_bestm.set("");
                        }
                    }

                    if (wa_ == PP) wa_ = NOUN;
                    if (wa_ == NOUN) {
                        hatte_schon_subjekt++;
                    }
                }

            }
        }

        subj.strip();
        prae.strip();
        prae.lower();
        obj.strip();
        fragewort.strip();
        wie_frage_objekt.strip();
        subj_hinten_dran.strip();

        /*
         * Verben: einheitliche Reihenfolge
         */
        jeliza::strings verben;
        util::split(prae, std::string(" "), verben);
        sort (verben.begin(), verben.end(), pred_lexicographical_compare);
        util::join(prae, std::string(" "), verben);

        jeliza::db_sentence ret;

        ret.subject = subj;
        ret.verb = prae;
        ret.object = obj + " " + subj_hinten_dran;
        ret.object.strip();
        if (!ret.object.size()) {
        	ret.object.set("nothing {}");
        }
        ret.quesword = fragewort.get_lower();
        ret.zusatz = wie_frage_objekt + adverbiale_bestimmungen;

        ret.print();

        return ret;

    }
    return jeliza::db_sentence();
}

/*
 * Liest die Wissensdatenbank ein
 */
void jeliza::jeliza_impl::init() {
	log("- Lade Datenbank ins RAM... ");

    jeliza_db.new_from(("lang_" + jeliza::get_lang_str() + "/jeliza-standard.xml").ref().c_str());
    jeliza_db.generate_python_db_file();

	log("- Datenbank erfolgreich geladen!");
	log("- Vorbereitung abgeschlossen!");
}

jeliza::db_manager& jeliza::jeliza_impl::get_db_manager() {
    return jeliza_db;
}

/*
 * Datenbank neu einlesen von ...
 */
void jeliza::db_manager::new_from(std::string file) {
    m_db.reset(parse(file));
}

/*
 * Lernt einen Satz, dh. formt ihn um (ich->du) und speichert die Datenbank
 */
void jeliza::jeliza_impl::learn (jeliza::strings anss) {
    if (force_answer_check) {
        log("Kann Satz nicht speichern wegen unbekanntem Wort.");
        return;
    }

	init();

	vector<jeliza::db_sentence> into_online_db;

    for (jeliza::strings::iterator it = anss.begin(); it != anss.end(); it++) {
        if (!offline_mode) {
            jeliza::async_download(online_db_new_sentence_entry_url.ref() + url_encode(KI_NAME + ": ").ref() + url_encode(*it).ref());
        }
        if (is_question(*it) != 0) {
            continue;
        }

        log("Speichere " + it->ref());
        it->set(util::umwandlung(*it));

        jeliza::strings temp;
        util::split(*it, std::string(" "), temp);

        if (temp.size() < 2) {
            log("- Satz hat zu wenig Woerter (< 2): " + it->ref());
            return;
        }

        shared_ptr<jeliza::db_sentence> dbs_ = jeliza::spo_split::get().get_spo(*it);
        jeliza::db_sentence dbs = *dbs_;

        jeliza_db.push(dbs);
        into_online_db.push_back(dbs);
    }
    log("Saving Database...");
    jeliza_db.save(("lang_" + jeliza::get_lang_str() + "/jeliza-standard.xml").ref().c_str());
    jeliza_db.generate_python_db_file();

    log("Sentence saved.");
//    if (use_online_db) {
        log("Using Online Database...");

        for (vector<jeliza::db_sentence>::iterator it = into_online_db.begin(); it != into_online_db.end(); it++) {
            if (!offline_mode) {
                jeliza::async_download(online_db_new_entry_url.ref() + url_encode(it->toXML()).ref());
            }
        }
        log("Online Database was downloaded...");
//    }
}

/*
 * Wrapper fuer *Datenbank*->push_back
 */
void jeliza::db_manager::push (db_sentence dbs) {
    m_db->push_back(dbs);
}

/*
 * Lernt einen Satz
 */
void jeliza::jeliza_impl::learn (jeliza::string fra) {
    log("learn (jeliza::string)");
	jeliza::strings anss;
	anss.push_back(fra);
	learn(anss);
}

PyObject* jelizacpp_update_prozent(PyObject *, PyObject *args) {
    float fl;

    if (!PyArg_ParseTuple(args, "f", &fl))
        return NULL;

    jeliza::update_prozent(fl);

    return Py_BuildValue("b", true);
}

PyObject* jelizacpp_say(PyObject *, PyObject *args) {
    char* c;

    if (!PyArg_ParseTuple(args, "s", &c))
        return NULL;

    speak(string(c));

    return Py_BuildValue("b", true);
}

PyObject* jelizacpp_is_question(PyObject *, PyObject *args) {
    char* c;

    if (!PyArg_ParseTuple(args, "s", &c))
        return NULL;

    return Py_BuildValue("b", jeliza::jeliza_impl::is_question(s(c)));
}

PyObject* jelizacpp_send(PyObject *, PyObject *args) {
    char* c;

    if (!PyArg_ParseTuple(args, "s", &c))
        return NULL;

    jeliza::comm_send(s(c));

    return Py_BuildValue("");
}

PyMethodDef jelizacpp_methods[] = {
    {"cpp_to_db_sentence",  jelizacpp_to_db_sentence, METH_VARARGS, "Converts a sentence to a 'db_sentence'."},
    {"cpp_hat_gleichen_wortstamm",  jelizacpp_hat_gleichen_wortstamm, METH_VARARGS, "Compares the 'Wortstamm' of two words."},
    {"cpp_update_prozent",  jelizacpp_update_prozent, METH_VARARGS, "Updates the procent label in the main window."},
    {"log",  jelizacpp_log, METH_VARARGS, "Replacement for print which prints the std::string in the log window, too."},
    {"cpp_www_wikipedia_de",  jelizacpp_search_in_wikipedia_with_newlines, METH_VARARGS, "Searches in Wikipedia."},
    {"cpp_listen_link_urls_get",  jelizacpp_listen_link_urls_get, METH_VARARGS, ""},
    {"cpp_bestimme_Wordtype_wrapper",  jelizacpp_bestimmeWordtypeWrapper, METH_VARARGS, ""},
    {"cpp_bestimme_Wordtype",  jelizacpp_bestimmeWordtype, METH_VARARGS, ""},
    {"cpp_umwandlung", jelizacpp_umwandlung, METH_VARARGS, ""},
    {"cpp_learn", jelizacpp_learn, METH_VARARGS, ""},
    {"display", jelizacpp_display, METH_VARARGS, ""},
    {"get_force_answer_word", jelizacpp_force_answer_word, METH_VARARGS, ""},
    {"get_force_answer", jelizacpp_force_answer, METH_VARARGS, ""},
    {"add_word", jelizacpp_add_word, METH_VARARGS, ""},
    {"cpp_say", jelizacpp_say, METH_VARARGS, ""},
    {"cpp_is_question", jelizacpp_is_question, METH_VARARGS, ""},
    {"send", jelizacpp_send, METH_VARARGS, ""},

    {NULL, NULL, 0, NULL}
};

PyObject* jelizacpp_listen_link_urls_get(PyObject *, PyObject *args) {
    unsigned int index = 1;

    if (!PyArg_ParseTuple(args, "i", &index))
        return NULL;

    index--;
    jeliza::string def;
    if (index < listenLinkUrls.size()) {
        std::string res = listenLinkUrls[index].ref();
        log("Resultierender Link:" + res);
        def = search_in_wikipedia_with_newlines(jeliza::string(res));
        log("Result:" + def.ref());
    }

    return Py_BuildValue("s", def.ref().c_str());
}

PyObject* jelizacpp_bestimmeWordtypeWrapper (PyObject *, PyObject *args) {
    unsigned int Wordtype;
    const char *command;
    bool atBegin = true;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    Wordtype = jeliza::spo_split::get().bestimmeWordtypeWrapper(jeliza::string(string(command)), atBegin);

    return Py_BuildValue("i", Wordtype);
}

PyObject* jelizacpp_bestimmeWordtype (PyObject *, PyObject *args) {
    unsigned int Wordtype;
    const char *command;
    bool atBegin = true;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    Wordtype = jeliza::spo_split::get().bestimmeWordtype(jeliza::string(string(command)), atBegin);

    return Py_BuildValue("i", Wordtype);
}

PyObject* jelizacpp_search_in_wikipedia_with_newlines(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::string def = search_in_wikipedia_with_newlines(jeliza::string(command));
    log("Result:" + def.ref());

    return Py_BuildValue("s", def.ref().c_str());
}



PyObject* jelizacpp_to_db_sentence(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    shared_ptr<jeliza::db_sentence> dbs = jeliza::spo_split::get().get_spo(jeliza::string(string(command)));
//    dbs->print();

    jeliza::db_sentence dbss;

    if (dbs->sub->size() > 0) {
        if (&(*dbs->sub)[0]) {
            dbss = (*dbs->sub)[0];
        }
    }

    dbs->print();

    return Py_BuildValue("(sssssssissbsssssssiss)", dbs->subject.get().c_str(), dbs->verb.get().c_str(),
            dbs->object.get().c_str(), dbs->prefix.get().c_str(), dbs->suffix.get().c_str(),
            dbs->feeling.get().c_str(), dbs->category.get().c_str(), dbs->priority, dbs->zusatz.get().c_str(),
            dbs->quesword.get().c_str(),

            dbs->sub->size() > 0, dbss.subject.get().c_str(), dbss.verb.get().c_str(),
            dbss.object.get().c_str(), dbss.prefix.get().c_str(), dbss.suffix.get().c_str(),
            dbss.feeling.get().c_str(), dbss.category.get().c_str(), dbss.priority, dbss.zusatz.get().c_str(),
            dbss.quesword.get().c_str());
}

PyObject* jelizacpp_log(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::log(string(command));

    return Py_BuildValue("s", command);
}

PyObject* jelizacpp_display(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::display_sentence(jeliza::string(string(command)).get_strip());

    return Py_BuildValue("b", 0);
}

PyObject* jelizacpp_umwandlung(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::string s;
    s.set(string(command));
    s = util::umwandlung(s);

    return Py_BuildValue("s", s.ref().c_str());
}
PyObject* jelizacpp_learn(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::string s;
    s.set(string(command));
    jeliza_impl_instance.learn(s);

    return Py_BuildValue("s", s.ref().c_str());
}

PyObject* jelizacpp_force_answer_word(PyObject *, PyObject *) {
    return Py_BuildValue("s", jeliza::force_answer_word.c_str());
}

PyObject* jelizacpp_force_answer(PyObject *, PyObject *) {
    return Py_BuildValue("s", jeliza::force_answer.ref().c_str());
}

PyObject* jelizacpp_add_word(PyObject *, PyObject *args) {
    const char *command;

    if (!PyArg_ParseTuple(args, "s", &command))
        return NULL;

    jeliza::jeliza_manager::get_current()->get().add_word(command[0]);

    return Py_BuildValue("b", false);
}



PyObject* jelizacpp_hat_gleichen_wortstamm(PyObject*, PyObject* args) {
    const char *s1;
    const char *s2;

    if (!PyArg_ParseTuple(args, "ss", &s1, &s2))
        return NULL;

    jeliza::string stamm1 = jeliza::spo_split::get().ermittle_wortstamm(jeliza::string(s1));
    jeliza::string stamm2 = jeliza::spo_split::get().ermittle_wortstamm(jeliza::string(s2));

//    jeliza::string prefixe_entfernt1 = jeliza::spo_split::get().entferne_prefixe(jeliza::string(s1));
//    jeliza::string prefixe_entfernt2 = jeliza::spo_split::get().entferne_prefixe(jeliza::string(s2));

    bool hat_gleichen_stamm = jeliza::spo_split::get().hat_gleichen_wortstamm(jeliza::string(s1), jeliza::string(s2),
            stamm1, stamm2);

    return Py_BuildValue("(b)", hat_gleichen_stamm);
}

double aktuelle_prozent = 0;
void prozent_plus(double p) {
    aktuelle_prozent += p;

    jeliza::update_prozent(aktuelle_prozent);
}

jeliza::DB jeliza::db_manager::get() {
    return *m_db;
}

/*
 * Formt die Datenbank ins Python-Format um und speichert sie
 */
void jeliza::db_manager::generate_python_db_file() {
    ofstream o2("temp/__init__.py");
    o2 << "import pyjdb" << endl;
    o2.close();


    ofstream o("temp/pyjdb.py");


    o << "db = []" << endl;

    for (unsigned int x = 0; x < m_db->size(); ++x) {
        db_sentence* it = new db_sentence((*m_db)[x]);
        it->strip();
        o << endl
          << "tempdbsent" << x << " = db_sentence()" << endl
          << "tempdbsent" << x << ".subject        = '" << it->subject.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".verb           = '" << it->verb.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".object         = '" << it->object.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".prefix         = '" << it->prefix.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".suffix         = '" << it->suffix.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".feeling        = '" << it->feeling.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".category       = '" << it->category.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".priority       = " << it->priority << "" << endl
          << "tempdbsent" << x << ".zusatz         = '" << it->zusatz.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".quesword       = '" << it->quesword.replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".all_ohne_muell = '" << jeliza::ohne_muell(it->genSentences_all(true)[0]).replace(string("'"), std::string("\"")).ref() << "'" << endl
          << "tempdbsent" << x << ".sub = db_sentence()" << endl
          << "tempdbsent" << x << ".has_sub = False" << endl
          << endl;

        o << endl;
        if (it->sub->size() > 0) {
            o << endl
              << "tempdbsent" << x << ".has_sub = True" << endl
              << "tempdbsent" << x << ".sub = db_sentence()" << endl
              << "tempdbsent" << x << ".sub.subject        = '" << (*it->sub)[0].subject.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.verb           = '" << (*it->sub)[0].verb.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.object         = '" << (*it->sub)[0].object.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.prefix         = '" << (*it->sub)[0].prefix.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.suffix         = '" << (*it->sub)[0].suffix.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.feeling        = '" << (*it->sub)[0].feeling.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.category       = '" << (*it->sub)[0].category.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.priority       = " << (*it->sub)[0].priority << "" << endl
              << "tempdbsent" << x << ".sub.zusatz         = '" << (*it->sub)[0].zusatz.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.quesword       = '" << (*it->sub)[0].quesword.replace(string("'"), std::string("\"")).ref() << "'" << endl
              << "tempdbsent" << x << ".sub.all_ohne_muell = '" << jeliza::ohne_muell((*it->sub)[0].genSentences_all(true)[0]).replace(string("'"), std::string("\"")).ref() << "'" << endl
              << endl;
        }
        o << endl
          << "db.append(tempdbsent" << x << ")" << endl;
    }


    o << endl << endl;
    o.flush();
    o.close();
}


/*
 * Antwortet auf eine Frage
 */
void jeliza::jeliza_impl::ask(jeliza::string frage) {
	clogger << "- Suche eine passende Antwort: " << endl;
    prozent_plus(1);

    jeliza_db.generate_python_db_file();

    ofstream o("temp/answer.tmp");
    o.close();

    jeliza::string ans;


    ofstream o2("temp/new_sentence.tmp");
    o2 << frage.ref();
    o2.close();
}

jeliza::string util::umwandlung (jeliza::string str) {
	str = str.replace("-", "");
	str = str.replace(";", "");
	str = str.replace("!", ".");
	str = str.replace_nocase_safe(s(","), s(" ,"));
	str = str.replace("?", " ucebckiundihwec");
	str = str.replace("ucebckiundihwec", "?");

	str = util::tausche(str, s("ich"), s("du"));
	str = util::tausche(str, s("mein"), s("dein"));
	str = util::tausche(str, s("meine"), s("deine"));
	str = util::tausche(str, s("meins"), s("deins"));
	str = util::tausche(str, s("meiner"), s("deiner"));
	str = util::tausche(str, s("meinem"), s("deinem"));
	str = util::tausche(str, s("mir"), s("dir"));
	str = util::tausche(str, s("mich"), s("dich"));
	str = util::tausche(str, s("user"), s("euer"));
	str = util::tausche(str, s("usere"), s("euere"));
	str = util::tausche(str, s("userer"), s("eurer"));
	str = util::tausche(str, s("useres"), s("euers"));

	str = util::tausche(str, s("am"), s("are"));
	str = util::tausche(str, s("my"), s("your"));
	str = util::tausche(str, s("mine"), s("yours"));
	str = util::tausche(str, s("myself"), s("yourself"));
	str = util::tausche(str, s("ourselves"), s("yourselves"));
	str = (" " + str + " ").replace_nocase_safe(s("our"), s("your")).get_strip();

	str = str.replace("wieso", "warum");
	str = str.replace("weshalb", "warum");
	str = (" " + str + " ").replace_nocase_safe(s("whom"), s("who")).get_strip();

	ifstream in("verbs.txt");
	vector<std::string> verbs;
	jeliza::string buffer;
	while (in) {
		getline(in, buffer.ref());
		buffer.strip();
		verbs.push_back(buffer.ref());
	}

	vector<std::string> ss;
	jeliza::strings ss2;
	split(str, std::string(" "), ss2);
	jeliza::string rep;

    for (jeliza::strings::iterator it = ss2.begin(); it != ss2.end(); it++) {
        ss.push_back(it->ref());
    }

    unsigned int noun_count = 0;
	for (unsigned int y = 0; y < ss.size(); y++) {
	    jeliza::string w(ss[y]);
		jeliza::string lower(w);
		lower.lower();
		w.strip();
		lower.strip();
		jeliza::Wordtype wa = jeliza::spo_split::get().bestimmeWordtype((y == 0) ? lower : w, y == 0);
		if (wa == jeliza::NOUN) noun_count++;

        bool is_verb = false;
        if (w.size() > 0) {
            is_verb = (wa == jeliza::VERB);
        }

        jeliza::string w2("-" + lower + "-");
        w2 = w2.replace(",-", "-,");
		if (is_verb && w2.contains("st-") && (w == lower || y == 0) && jeliza::get_lang() == jeliza::GERMAN) {
            jeliza::string verb2("-" + lower + "-");
            verb2 = verb2.replace(",-", "-,");
            if (verb2.contains("-bist-")) {
    			rep += "bin";
    			rep += " ";
    			continue;
            }
            if (verb2.contains("-hast-")) {
    			rep += "habe";
    			rep += " ";
    			continue;
            }
            verb2 = verb2.replace("willst", "will");
            verb2 = verb2.replace_nocase_safe(s("sst-"), s("ssst-"));
            verb2 = verb2.replace("st-", "e-");
            verb2 = verb2.replace("zt-", "ze-");
            verb2 = verb2.replace("ee-", "hoire-");
            verb2 = verb2.replace("hoire-", "e-");
            verb2 = verb2.replace("-", "");
        	if (verb2 == "ie") verb2.set("ist");
			rep += verb2.ref();
			rep += " ";
		}
		else if (is_verb && (w == lower || y == 0) && jeliza::get_lang() == jeliza::GERMAN) {
            jeliza::string verb2("-" + lower + "-");
            verb2 = verb2.replace(",-", "-,");
            if (verb2.contains("-bin-")) {
    			rep += "bist";
    			rep += " ";
    			continue;
            }
            if (verb2.contains("-habe-")) {
    			rep += "hast";
    			rep += " ";
    			continue;
            }
            verb2 = verb2.replace_nocase_safe(s("will"), s("willst"));
            verb2 = verb2.replace("nte-", "ntee-");
            verb2 = verb2.replace("lte-", "ltee-");
            verb2 = verb2.replace("mte-", "mtee-");
            verb2 = verb2.replace("rte-", "rtee-");
            verb2 = verb2.replace("pte-", "ptee-");
            verb2 = verb2.replace("bte-", "btee-");
            verb2 = verb2.replace("kte-", "ktee-");
            verb2 = verb2.replace("tte-", "ttee-");
            verb2 = verb2.replace("e-", "st-");
            verb2 = verb2.replace("zst-", "zt-");
            verb2 = verb2.replace("-", "");
			rep += verb2.ref();
			rep += " ";
		}
		else {
			w = w.replace("-", "");

			if (noun_count == 1) {
				if (w.get_lower() == "you") w.set("I");
				else if (w.get_lower() == "i") w.set("you");
				else if (w.get_lower() == "me") w.set("you");
			}
			else {
				if (w.get_lower() == "you") w.set("me");
				else if (w.get_lower() == "i") w.set("you");
				else if (w.get_lower() == "me") w.set("you");
			}

			rep += w.replace("-", "").ref();
			rep += " ";
		}

	}

    str = rep;
	str = str.replace(string("-"), std::string(""));
	str = str.replace(string(" ,"), std::string(","));
//	str = util::tausche(str, s("heisse"), s("heisst"));
	str = str.replace(string("eise"), std::string("eisse"));
	str = str.replace(string("wist"), std::string("wie"));
//	str = util::tausche(str, s("bin"), s("bist"));


//	str = util::tausche(str, s("habst"), s("hast"));

	return (jeliza::string(str));
}



#endif
