#ifndef JELIZA_SENTENCE
#define JELIZA_SENTENCE 1


/*
 * This is part of JEliza 2.3.
 * Copyright 2006 by Tobias Schulz
 * WWW: http://jeliza.ch.to/
 *
 * JEliza is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * JEliza is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU GPL
 * along with JEliza (file gpl.txt) ; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "defs.h"

using namespace std;


string jeliza::string::get() {
    return *m_sent;
}

string jeliza::string::get() const {
    return *m_sent;
}

string& jeliza::string::ref() {
    return *m_sent;
}

string jeliza::string::ref() const {
    return *m_sent;
}

jeliza::string& jeliza::string::set(const std::string& str) {
    m_sent.reset(new std::string(str.c_str()));
    return (*this);
}
jeliza::string& jeliza::string::set(const jeliza::string& str) {
    m_sent.reset(new std::string(str.get().c_str()));
    return (*this);
}
jeliza::string& jeliza::string::set(const char* str) {
    m_sent.reset(new std::string(str));
    return (*this);
}

unsigned int jeliza::string::size() const {
    return (m_sent->size());
}

unsigned int jeliza::string::length() const {
    return (size());
}

int jeliza::string::find(std::string s) {
    return (ref().find(s));
}
int jeliza::string::find(const char s) {
    return (ref().find(s));
}
jeliza::string jeliza::string::substr(int x, int y) {
    jeliza::string ret = *this;
    ret.ref() = ret.ref().substr(x, y);
    return ret;
}

char& jeliza::string::operator[] (int index) {
    if (index < 0) {
        index = ref().size() - index;
    }
    return (ref()[index]);
}

void jeliza::string::lower() {
    if (&(*m_sent)) {
        if (size() == 0) {
            return;
        }
    }
    for (std::string::size_type x = 0; x < ref().size(); x++) {
		ref()[x] = tolower(ref()[x]);
	}
}
void jeliza::string::upper() {
    if (&(*m_sent)) {
        if (size() == 0) {
            return;
        }
    }
    for (std::string::size_type x = 0; x < ref().size(); x++) {
		ref()[x] = toupper(ref()[x]);
	}
}

jeliza::string jeliza::string::get_lower() const {
    jeliza::string ret = *this;
    ret.lower();
    return (ret);
}
jeliza::string jeliza::string::get_upper() const {
    jeliza::string ret = *this;
    ret.upper();
    return (ret);
}

jeliza::string jeliza::string::get_strip() const {
    jeliza::string ret = *this;
    ret.strip();
    return (ret);
}

jeliza::string jeliza::string::replace(const jeliza::string rep, const jeliza::string wit) {
	int pos;
	while (true) {
		pos = ref().find(rep.get());
		if (pos == -1) {
			break;
		} else {
			ref().erase(pos, rep.get().size());
			ref().insert(pos, wit.get());
		}
	}
	return (*this);
}
jeliza::string jeliza::string::replace(const jeliza::string rep, const std::string wit) {
    return (replace (rep, jeliza::string(wit)));
}
jeliza::string jeliza::string::replace(const std::string rep, const std::string wit) {
    return (replace (jeliza::string(rep), jeliza::string(wit)));
}
jeliza::string jeliza::string::replace_nocase(const jeliza::string rep, const jeliza::string wit) {
	int pos;
	while (true) {
	    jeliza::string low;
        (low.set(get())).lower();
		pos = low.find(rep.get());
		if (pos == -1) {
			break;
		} else {
			ref().erase(pos, rep.get().size());
			ref().insert(pos, wit.get());
		}
	}
	return (*this);
}
jeliza::string jeliza::string::replace_nocase_safe(const jeliza::string rep, const jeliza::string wit) {
	replace_nocase(rep, jeliza::string("indhwszterbvdtshsbcuebdfjsiahbsdsagsvbxybwlcombxgfsszsv"));
	replace_nocase(jeliza::string("indhwszterbvdtshsbcuebdfjsiahbsdsagsvbxybwlcombxgfsszsv"), wit);
	return (*this);
}
jeliza::string jeliza::string::replace_first(const jeliza::string rep, const jeliza::string wit) {
	int pos;
    pos = ref().find(rep.get());
    if (pos != -1) {
        ref().erase(pos, rep.get().size());
        ref().insert(pos, wit.get());
	}
	return (*this);
}
jeliza::string jeliza::string::replace_first_nocase(const jeliza::string rep, const jeliza::string wit) {
	int pos;
    jeliza::string low;
    (low.set(get())).lower();
    jeliza::string low_rep;
    (low_rep.set(rep)).lower();
    pos = low.find(low_rep.get());
    if (pos != -1) {
        ref().erase(pos, rep.get().size());
        ref().insert(pos, wit.get());
	}
	return (*this);
}
jeliza::string jeliza::string::replace_first(const jeliza::string rep, const std::string wit) {
    return (replace_first (rep, jeliza::string(wit)));
}
jeliza::string jeliza::string::replace_first(const std::string rep, const std::string wit) {
    return (replace_first (jeliza::string(rep), jeliza::string(wit)));
}
bool jeliza::string::contains (const jeliza::string rep) {
    int pos = ref().find(rep.get());
    if (pos == -1) {
		return 0;
	}
	return 1;
}
bool jeliza::string::contains (const char* rep) {
    int pos = ref().find(std::string(rep));
    if (pos == -1) {
		return 0;
	}
	return 1;
}

bool jeliza::string::startswith (const jeliza::string rep) {
    int pos = ref().find(rep.get());
    if (pos == 0) {
		return true;
	}
	return false;
}
bool jeliza::string::startswith (const char* rep) {
    int pos = ref().find(std::string(rep));
    if (pos == 0) {
		return true;
	}
	return false;
}
bool jeliza::string::endswith (const jeliza::string rep) {
    int pos = ref().find(rep.get()) + rep.size();
    if (pos >= ref().size() - 1) {
		return true;
	}
	return false;
}
bool jeliza::string::endswith (const char* rep) {
    return endswith(jeliza::string(rep));
}
void jeliza::string::strip() {
/*    for (int y = 0; y < 2; y++) {
        if (m_sent->length()) {
            while (m_sent->length() > 0) {
                if ((*m_sent)[0] != ' ' && (*m_sent)[0] != '\n' && (*m_sent)[0] != '\r') {
                    break;
                }
                m_sent->erase(0, 1);
            }


            reverse(m_sent->begin(), m_sent->end());
        }
    }*/

    if (m_sent->length()) {
        while (m_sent->length() > 0) {
            if ((*m_sent)[0] != ' ' && (*m_sent)[0] != '\n' && (*m_sent)[0] != '\r') {
                break;
            }
            m_sent->erase(0, 1);
        }

        int len = m_sent->length() - 1;
        while (len > -1) {
            if ((*m_sent)[len] != ' ' && (*m_sent)[len] != '\n' && (*m_sent)[len] != '\r') {
                break;
            }
            m_sent->erase(len, 1);
            --len;
        }
    }
}

jeliza::string jeliza::string::toASCII() {
    jeliza::string ret = *this;
    jeliza::string& all = ret;

    all = all.replace("Ã¼", "ue");
	all = all.replace("Ã", "ss");
	all = all.replace("Ã¤", "ae");
	all = all.replace("ü", "ue");
	all = all.replace("ß", "ss");
	all = all.replace("ä", "ae");
	all = all.replace("ö", "oe");
	all = all.replace("Ü", "Ue");
	all = all.replace("Ä", "Ae");
	all = all.replace("Ö", "Oe");
	all = all.replace("Ã¢ÂÂ", "\"");
	all = all.replace("Ã¢ÂÂ", "\"");
	all = all.replace("&lt;/br&gt;", " ");

	jeliza::string ascii(" \n\r!\"#$%&'()*+,-./0123456789:;<=>?    @ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_    `abcdefghijklmnopqrstuvwxyz{|}~");

	string allAscii;
	for (int x = 0; x < all.size(); x++) {
		char array[2];
		array[0] = all[x];
		array[1] = '\0';
		string y(array);
		if (ascii.contains(y.ref().c_str())) {
			allAscii += y;
		}
	}
    ret.set(allAscii);

	return (ret);
}

jeliza::string& operator+ (jeliza::string s1, jeliza::string s2) {
    (*s1.m_sent) += s2.ref();
    return s1;
}
jeliza::string& operator+ (jeliza::string s1, std::string s2) {
    (*s1.m_sent) += s2;
    return s1;
}
jeliza::string& operator+ (jeliza::string s1, const char* s2) {
    (*s1.m_sent) += s2;
    return s1;
}
jeliza::string operator+ (string s2, jeliza::string s1) {
    jeliza::string temp(s2);
    temp += s1;
    return temp;
}
jeliza::string& operator+ (const char* s2, jeliza::string s1) {
    (*s1.m_sent) = std::string(s2) + (*s1.m_sent);
    return s1;
}


jeliza::string& operator+= (jeliza::string& s1, const jeliza::string s2) {
    (*s1.m_sent) += (*s2.m_sent);
    return s1;
}
jeliza::string& operator+= (jeliza::string& s1, std::string s2) {
    (*s1.m_sent) += s2;
    return s1;
}
jeliza::string& operator+= (jeliza::string& s1, const char* s2) {
    (*s1.m_sent) += s2;
    return s1;
}

/*jeliza::string& jeliza::string::operator= (string sent) {
    set (sent);
    return (*this);
}*/
/*jeliza::string& jeliza::string::operator= (const char* sent) {
    set (string(sent));
    return (*this);
}*/
jeliza::string& jeliza::string::operator= (const jeliza::string& sent) {
    set (sent);
    return (*this);
}

bool jeliza::string::operator== (const jeliza::string& s2) const {
    return (equal(s2));
}
bool jeliza::string::operator== (const std::string& s2) const {
    return (equal(s2));
}
bool jeliza::string::operator== (const char* s2) const {
    return (equal(s2));
}

bool jeliza::string::equal (const jeliza::string& s2) const {
    return ((*m_sent) == s2.ref());
}
bool jeliza::string::equal (const std::string& s2) const {
    return ((*m_sent) == s2);
}
bool jeliza::string::equal (const char* s2) const {
    return ((*m_sent) == std::string(s2));
}

bool jeliza::string::operator< (const string& s) const {
    return (size() < s.size());
}


/*jeliza::strings& operator<< (jeliza::strings& a1, const jeliza::strings& a2) {
    for (int x = 0; x < a2.size(); x++) {
        a1.push_back(a2[x]);
    }
    return a1;
}
jeliza::strings& operator<< (jeliza::strings& a1, int*) {
    a1.erase(a1.begin(), a1.end());
    return a1;
}*/

string::iterator jeliza::string::begin() const {
    return m_sent->begin();
}
string::iterator jeliza::string::end() const {
    return m_sent->end();
}




#endif
