#include "defs.h"


/*
 * This is part of JEliza 2.3.
 * Copyright 2006 by Tobias Schulz
 * WWW: http://jeliza.ch.to/
 *
 * JEliza is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later
 * version.
 *
 * JEliza is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU GPL
 * along with JEliza (file gpl.txt) ; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#include <Python.h>

#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <queue>
#include <sstream>

#include <string>
#include <list>

#include <time.h>
#include <vector>
#include <unistd.h>
#include <getopt.h>

#include <algorithm>

#include <sys/time.h>

#include <stdlib.h>




#include <QtGui/QApplication>
#include <QtGui/QPushButton>

#include <QtGui/QApplication>
#include <QtUiTools/QUiLoader>
#include <QtGui/QMainWindow>
#include <QtGui/QWidget>
#include <QtCore/QFile>
#include <QtCore/QMutexLocker>
#include <QtCore/QMutex>
#include <QtCore/QtCore>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <QtGui/QPainter>
#include <QtGui/QScrollArea>

#include "ui_jeliza.h"
#include "ui_options.h"
#include "ui_db_edit.h"
#include "ui_help.h"
#include "ui_changelog.h"
#include "ui_startup.h"
#include "ui_ee.h"
#include "ui_connect.h"
#include "ui_view.h"

using namespace std;

extern auto_ptr<QString> jelizathread1_logs;
extern auto_ptr<QString> jelizathread1_talking;

void exportiere_db(string, string);
void make_connection();
void check_new_version();

namespace jeliza {
	void display_sentence(jeliza::string);
	void comm_new(jeliza::string);
	void log(std::string);
	void update_prozent(float);
}

void make_connection();
void exportiere_db(string, string);

class QJElizaApplication : public QApplication {
    Q_OBJECT

public:
    QJElizaApplication(int, char**);
//private:
//    QJElizaApplication(const QJElizaApplication&);

public slots:
    void exitNow();
};

class JElizaWindow : public QMainWindow {
    Q_OBJECT

public:
    bool waitForStatusbar;


    JElizaWindow(QWidget *parent = 0);

    Ui::jelizaWindow* ui_form;
    Ui::Dialog* ui_dia;


    void showWindowNormal();
    void showWindowFullscreen();
    void showWindowEE();

signals:
    void signalAfterExport(QString);

public slots:
    void hideStartupDialog();
    void slotStatusbarUpdated(QString);
    void on_normalscreen_clicked();
    void on_fullscreen_triggered();
    void on_pushButton_clicked();
    void on_lineEdit_returnPressed();
    void on_actionGespr_ch_triggered();
    void on_actionDatenbank_triggered();
    void on_actionWissen_importieren_triggered();
    void on_actionWissen_exportieren_triggered();
    void on_actionSemantisches_Netz_triggered();
    void on_actionEinstellungen_triggered();
    void on_actionInfo_triggered();
    void on_will_show_log_toggled(bool);
    void slotAfterExport(QString);

    void on_w_verb_clicked();
    void on_w_nomen_clicked();
    void on_w_name_clicked();
    void on_w_adj_clicked();
    void on_w_adv_clicked();
    void on_w_pronomen_clicked();
    void on_w_quesword_clicked();
    void on_w_prep_clicked();
    void on_w_adv_best_clicked();
    void on_w_break_clicked();
    void on_w_interj_clicked();
    void on_w_interj2_clicked();

    void on_male_clicked();
    void on_female_clicked();
    void on_thing_clicked();
    void on_is_noun_clicked();
    void on_not_is_noun_clicked();

friend void exportiere_db(string, string);
};

class Dialog1 : public QDialog {
    Q_OBJECT

public:
    Ui::jelizaWindow* ui_form;
    Ui::Dialog* ui_dia;

signals:
    void signal_on_download_finished();
    void signal_on_import_finished();

public slots:
//    void on_fullscreen_clicked();

    void on_clear_screen_clicked();
    void on_db_download_clicked();
    void on_proxy_ok_clicked();
    void on_db_edit_clicked();
    void on_edit_partners_clicked();
    void on_mode_www_toggled();
    void on_mode_offline_toggled();
    void on_speech_toggled();
    void on_arin_load_clicked();
    void on_arin_save_clicked();
    void on_edit_modules_clicked();
    void on_access_online_db_toggled();
    void on_download_finished();
    void on_import_finished();

friend void jeliza::comm_new(jeliza::string);
};

class Dialog2 : public QDialog {
    Q_OBJECT

public:
    Ui::jelizaWindow* ui_form;
    Ui::db_edit* ui_dia;

    string full_name;
    string file;

public slots:
    void on_save_clicked();
    void on_hide_clicked();

    void init_editor();
};

class Dialog3 : public QDialog {
    Q_OBJECT

public:
    Ui::jelizaWindow* ui_form;
    Ui::HelpDialog* ui_dia;

public slots:
    void on_changelog_clicked();
};

class Dialog4 : public QDialog {
    Q_OBJECT

public:
    Ui::jelizaWindow* ui_form;
    Ui::ChangelogDialog* ui_dia;

    void init_changelog();
};

class Dialog5 : public QDialog {
    Q_OBJECT

public:
    void updateStatusBar(float fl);

    Ui::jelizaWindow* ui_form;
    Ui::Startup* ui_dia;

signals:
    void signalUpdateStatusbar(QString);

};

class Dialog6 : public QDialog {
    Q_OBJECT

public:
    void updateStatusBar(float fl);

    Ui::jelizaWindow* ui_form;
    Ui::EasterEgg* ui_dia;
};

class Dialog7 : public QDialog {
    Q_OBJECT
};

class Dialog8 : public QDialog {
    Q_OBJECT

public:
    void updateStatusBar(float fl);

    Ui::jelizaWindow* ui_form;
    Ui::ViewDialog* ui_dia;
};

class AnswerHelper : public QObject {
    Q_OBJECT

friend void jeliza::display_sentence(jeliza::string);
friend void jeliza::comm_new(jeliza::string);
friend void jeliza::log(std::string);
friend void jeliza::update_prozent(float);
friend void make_connection();
friend void check_new_version();

public slots:
	void showMsgWindow();
	void hideMsgWindow();
	void showMsgGenusWindow();
	void hideMsgGenusWindow();
	void showMsgNounOrNotWindow();
	void hideMsgNounOrNotWindow();
    void slotEverythingReady();
    void close_view();
	void slotNewVersionOnline(QString);

signals:
	void signalNewVersionOnline(QString);

    void signalTalkingScrollEnd(QString);
    void signalAnswer(QString);
    void signalUpdateStatusbar(QString);
    void signalUpdateProgressbar(int);
    void signalHideProgressbar();
    void signalShowProgressbar();
    void signalLog(QString);
    void signalLogScrollEnd(QString);
    void signalChangeWindowTitle(QString);
    void signalChangeHelpWindowText1(QString);
    void signalChangeHelpWindowText2(QString);
    void signalMsg(QString);
    void signalMsgGenus(QString);
    void signalMsgNounOrNot(QString);
    void signalShowMsgWindow();
    void signalHideMsgWindow();
    void signalShowMsgGenusWindow();
    void signalHideMsgGenusWindow();
    void signalShowMsgNounOrNotWindow();
    void signalHideMsgNounOrNotWindow();
    void everythingReady();
};


class SemanticWidget : public QWidget {
    Q_OBJECT

public:
    SemanticWidget(QDialog *parent = 0);

    QSize minimumSizeHint() const;
    QSize sizeHint() const;
    void draw();

protected:
    QGridLayout *gridLayout;
    QDialog* dia;
    int xmax;
    int ymax;
    QPixmap bg;
    bool schon_gezeichnet;

    void paintEvent(QPaintEvent *event);
};

class ConnectDialog : public QDialog {
    Q_OBJECT

public:
    Ui::ConnectDialog* from;
};

