/* copyright © by Stefan Voitel, 2003 - all rights reserved                   */
/* eMail : stefan.voitel@winways.de                                           */
/* access system mixer                                                        */

#ifndef __CLASS_MMCONTROL_INCLUDED__
#define __CLASS_MMCONTROL_INCLUDED__

#include <windows.h>
#include <mmsystem.h>
#include "cMMControl.h"

cMMControl::cMMControl (void) {
hMixer      = NULL;
mcSwitch    = NULL;
mcFader     = NULL;
}

cMMControl::~cMMControl (void) {
mixerClose ((HMIXER) hMixer);
}

// ------------------------------------------------

bool cMMControl::Open (tSource src,tDestination dst,tFader fd,tSwitch sw) {
bool success = false;

if (mixerGetNumDevs()) {
   MIXERCAPS  Caps;

   if (mixerGetDevCaps (0,&Caps,sizeof(Caps)) == MMSYSERR_NOERROR) {
      if (mixerOpen ((HMIXER*) &hMixer,0,0,0,0) == MMSYSERR_NOERROR) {
         UINT  LineID = 0,Count = Caps.cDestinations;

         if (GetDestination (LineID,Count,dst)) {
            if (GetSource (LineID,Count,src)) {
               if (GetControls (LineID,Count,sw,fd))
                  success = true;
               }
            }
         }
      }
   }
return success;
}

//---------------------------------------------------------------------------

bool cMMControl::GetDestination (UINT &id,UINT &n,tDestination dst) {
bool        success = false;
MIXERLINE   mlDest;
UINT        cnt;

for (cnt = 0 ; cnt < n && ! success ; cnt++) {
	mlDest.cbStruct      = sizeof(MIXERLINE);
	mlDest.dwDestination = cnt;

	if  (mixerGetLineInfo(0,&mlDest,MIXER_GETLINEINFOF_DESTINATION)
																		== MMSYSERR_NOERROR) {
      if (mlDest.dwComponentType == (DWORD) dst) {
         n  = mlDest.cConnections;
         id = cnt;
         success =  true;
         }
      }
   }
return success;
}

//---------------------------------------------------------------------------

bool cMMControl::GetSource (UINT &id,UINT &n,tSource src) {
bool        success = false;
MIXERLINE   mlSource;
UINT        cnt;

if (src != srcNone) {
   for (cnt = 0; cnt < n && ! success; cnt++) {
      mlSource.cbStruct      = sizeof(MIXERLINE);
      mlSource.dwDestination = id;
      mlSource.dwSource      = cnt;

      if (mixerGetLineInfo(0,&mlSource,MIXER_GETLINEINFOF_SOURCE)
                                                            == MMSYSERR_NOERROR) {
         if (mlSource.dwComponentType == (DWORD) src) {
            n  = mlSource.cControls;
            id = mlSource.dwLineID;
            success = true;
            }
         }
      }
   }
else {
   id += 0xFFFF0000;
   success = true;
   }
return success;
}

//---------------------------------------------------------------------------

bool cMMControl::GetControls(UINT id,UINT n,tSwitch sw,tFader fd) {
MIXERLINECONTROLS    mxlc;
tagMIXERCONTROLA        *pctrl;
bool                 success = false;

if ((pctrl = (tagMIXERCONTROLA*)LocalAlloc(LPTR,n*sizeof(tagMIXERCONTROLA))) != NULL) {
   mxlc.cbStruct       = sizeof(mxlc);
   mxlc.dwLineID       = id;
   mxlc.cControls      = n;
   mxlc.cbmxctrl       = sizeof(MIXERCONTROL);
   mxlc.pamxctrl       = pctrl;
   mxlc.dwControlID    = 0;
   mxlc.dwControlType  = 0;

   if (mixerGetLineControls (hMixer,&mxlc,MIXER_GETLINECONTROLSF_ALL)
                                                         == MMSYSERR_NOERROR) {
      UINT cnt;

      for (cnt = 0; cnt < n; cnt ++) {
         if (pctrl[cnt].dwControlType == (DWORD) sw &&
                  ! (pctrl[cnt].fdwControl & MIXERCONTROL_CONTROLF_DISABLED)) {

            mcSwitch = (MIXERCONTROL*) LocalAlloc (LPTR,sizeof (MIXERCONTROL));

            if (mcSwitch != NULL) {
               CopyMemory (mcSwitch,&pctrl[cnt],sizeof(MIXERCONTROL));

               MIXERCONTROLDETAILS mcds;
               mcds.cbStruct       = sizeof(MIXERCONTROLDETAILS);
               mcds.dwControlID    = mcSwitch->dwControlID;
               mcds.cChannels      = 1;
               mcds.cMultipleItems = mcSwitch->cMultipleItems;
               mcds.cbDetails      = sizeof(mdSwitch);
               mcds.paDetails      = &mdSwitch;

               mixerGetControlDetails (hMixer,&mcds,
                                                MIXER_GETCONTROLDETAILSF_VALUE);
               success = true;
               }
           }
         else if (pctrl[cnt].dwControlType == (DWORD) fd &&
                  ! (pctrl[cnt].fdwControl & MIXERCONTROL_CONTROLF_DISABLED)) {

            mcFader = (MIXERCONTROL*) LocalAlloc (LPTR,sizeof (MIXERCONTROL));

            if (mcFader != NULL) {
               CopyMemory (mcFader,&pctrl[cnt],sizeof(MIXERCONTROL));
               MIXERCONTROLDETAILS mcdf;
               mcdf.cbStruct       = sizeof(MIXERCONTROLDETAILS);
               mcdf.dwControlID    = mcFader->dwControlID;
               mcdf.cChannels      = 1;
               mcdf.cMultipleItems = mcFader->cMultipleItems;
               mcdf.cbDetails      = sizeof(mdFader);
               mcdf.paDetails      = &mdFader;

               mixerGetControlDetails (hMixer,&mcdf,
                                                MIXER_GETCONTROLDETAILSF_VALUE);
               success = true;
               }
            }
         }
      }
   LocalFree (pctrl);
   }
return success;
}

//---------------------------------------------------------------------------

bool cMMControl::Fade (UINT percent) {
bool        success = false;

if (mcFader != NULL) {
   MIXERCONTROLDETAILS           mcd;
   MIXERCONTROLDETAILS_UNSIGNED  mdu;

   mcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
   mcd.dwControlID    = mcFader->dwControlID;
   mcd.cChannels      = 1;
   mcd.cMultipleItems = mcFader->cMultipleItems;
   mcd.cbDetails      = sizeof(mdu);
   mcd.paDetails      = &mdu;

   #if defined (__BORLANDC__)
   double f = ((mcFader->Bounds.s1.dwMaximum / 100) * percent);
   #else
   double f = ((mcFader->Bounds.dwMaximum / 100) * percent);
   #endif

   mdu.dwValue = (DWORD) f;

   if (mixerSetControlDetails (hMixer,&mcd,MIXER_SETCONTROLDETAILSF_VALUE)
                                                       == MMSYSERR_NOERROR) {
      success = true;
      }
   }
return success;
}

//---------------------------------------------------------------------------

bool cMMControl::Switch (bool enable) {
bool        success = false;

if (mcFader != NULL) {
   MIXERCONTROLDETAILS           mcd;
   MIXERCONTROLDETAILS_BOOLEAN   mdb;

   mcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
   mcd.dwControlID    = mcSwitch->dwControlID;
   mcd.cChannels      = 1;
   mcd.cMultipleItems = mcSwitch->cMultipleItems;
   mcd.cbDetails      = sizeof(mdb);
   mcd.paDetails      = &mdb;

   mdb.fValue = ! enable;

   if (mixerSetControlDetails (hMixer,&mcd,MIXER_SETCONTROLDETAILSF_VALUE)
                                                         == MMSYSERR_NOERROR) {
      success = true;
      }
   }
return success;
}

//---------------------------------------------------------------------------

void cMMControl::Reset (void) {
MIXERCONTROLDETAILS   mcd;
mcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);

if (mcFader != NULL) {
   mcd.dwControlID    = mcFader->dwControlID;
   mcd.cMultipleItems = mcFader->cMultipleItems;
   mcd.cChannels      = 1;
   mcd.cbDetails      = sizeof(mdFader);
   mcd.paDetails      = &mdFader;
   mixerSetControlDetails (hMixer,&mcd,MIXER_SETCONTROLDETAILSF_VALUE);
   }

if (mcSwitch != NULL) {
   mcd.dwControlID    = mcSwitch->dwControlID;
   mcd.cMultipleItems = mcSwitch->cMultipleItems;
   mcd.cChannels      = 1;
   mcd.cbDetails      = sizeof(mdSwitch);
   mcd.paDetails      = &mdSwitch;
   mixerSetControlDetails (hMixer,&mcd,MIXER_SETCONTROLDETAILSF_VALUE);
   }
}
#endif // (__CLASS_MMCONTROL_INCLUDED__)
// EOF
